@@ -14,7 +14,6 @@ where
 import Data.List
 import Data.Maybe (fromMaybe)
 import Data.Monoid
--- import Data.Text (Text)
 import qualified Data.Text as T
 import Test.HUnit
 -- import qualified Data.Map as M
@@ -24,7 +23,7 @@ import Hledger.Utils
 
 
 -- characters that may not be used in a non-quoted commodity symbol
-nonsimplecommoditychars = "0123456789-+.@;\n \"{}=" :: [Char]
+nonsimplecommoditychars = "0123456789-+.@*;\n \"{}=" :: [Char]
 
 quoteCommoditySymbolIfNeeded s | any (`elem` nonsimplecommoditychars) (T.unpack s) = "\"" <> s <> "\""
                                | otherwise = s
