@@ -3,6 +3,7 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-|
 
 Date parsing and utilities for hledger.
@@ -72,6 +73,7 @@ import Prelude ()
 import Prelude.Compat
 import Control.Monad
 import Data.List.Compat
+import Data.Default
 import Data.Maybe
 import Data.Text (Text)
 import qualified Data.Text as T
@@ -86,8 +88,8 @@ import Data.Time.Calendar.OrdinalDate
 import Data.Time.Clock
 import Data.Time.LocalTime
 import Safe (headMay, lastMay, readMay)
-import Text.Megaparsec
-import Text.Megaparsec.Text
+import Text.Megaparsec.Compat
+import Text.Megaparsec.Perm
 import Text.Printf
 
 import Hledger.Data.Types
@@ -165,9 +167,15 @@ spansSpan spans = DateSpan (maybe Nothing spanStart $ headMay spans) (maybe Noth
 -- >>> t (Weeks 2) "2008/01/01" "2008/01/15"
 -- [DateSpan 2007/12/31-2008/01/13,DateSpan 2008/01/14-2008/01/27]
 -- >>> t (DayOfMonth 2) "2008/01/01" "2008/04/01"
--- [DateSpan 2008/01/02-2008/02/01,DateSpan 2008/02/02-2008/03/01,DateSpan 2008/03/02-2008/04/01]
+-- [DateSpan 2007/12/02-2008/01/01,DateSpan 2008/01/02-2008/02/01,DateSpan 2008/02/02-2008/03/01,DateSpan 2008/03/02-2008/04/01]
+-- >>> t (WeekdayOfMonth 2 4) "2011/01/01" "2011/02/15"
+-- [DateSpan 2010/12/09-2011/01/12,DateSpan 2011/01/13-2011/02/09,DateSpan 2011/02/10-2011/03/09]
 -- >>> t (DayOfWeek 2) "2011/01/01" "2011/01/15"
--- [DateSpan 2011/01/04-2011/01/10,DateSpan 2011/01/11-2011/01/17]
+-- [DateSpan 2010/12/28-2011/01/03,DateSpan 2011/01/04-2011/01/10,DateSpan 2011/01/11-2011/01/17]
+-- >>> t (DayOfYear 11 29) "2011/10/01" "2011/10/15"
+-- [DateSpan 2010/11/29-2011/11/28]
+-- >>> t (DayOfYear 11 29) "2011/12/01" "2012/12/15"
+-- [DateSpan 2011/11/29-2012/11/28,DateSpan 2012/11/29-2013/11/28]
 --
 splitSpan :: Interval -> DateSpan -> [DateSpan]
 splitSpan _ (DateSpan Nothing Nothing) = [DateSpan Nothing Nothing]
@@ -177,8 +185,10 @@ splitSpan (Weeks n)      s = splitspan startofweek    (applyN n nextweek)    s
 splitSpan (Months n)     s = splitspan startofmonth   (applyN n nextmonth)   s
 splitSpan (Quarters n)   s = splitspan startofquarter (applyN n nextquarter) s
 splitSpan (Years n)      s = splitspan startofyear    (applyN n nextyear)    s
-splitSpan (DayOfMonth n) s = splitspan (nthdayofmonthcontaining n) (applyN (n-1) nextday . nextmonth) s
+splitSpan (DayOfMonth n) s = splitspan (nthdayofmonthcontaining n) (nthdayofmonth n . nextmonth) s
+splitSpan (WeekdayOfMonth n wd) s = splitspan (nthweekdayofmonthcontaining n wd) (advancetonthweekday n wd . nextmonth) s
 splitSpan (DayOfWeek n)  s = splitspan (nthdayofweekcontaining n)  (applyN (n-1) nextday . nextweek)  s
+splitSpan (DayOfYear m n) s= splitspan (nthdayofyearcontaining m n) (applyN (n-1) nextday . applyN (m-1) nextmonth . nextyear) s
 -- splitSpan (WeekOfYear n)    s = splitspan startofweek    (applyN n nextweek)    s
 -- splitSpan (MonthOfYear n)   s = splitspan startofmonth   (applyN n nextmonth)   s
 -- splitSpan (QuarterOfYear n) s = splitspan startofquarter (applyN n nextquarter) s
@@ -256,8 +266,8 @@ earliest (Just d1) (Just d2) = Just $ min d1 d2
 
 -- | Parse a period expression to an Interval and overall DateSpan using
 -- the provided reference date, or return a parse error.
-parsePeriodExpr :: Day -> Text -> Either (ParseError Char Dec) (Interval, DateSpan)
-parsePeriodExpr refdate = parsewith (periodexpr refdate <* eof)
+parsePeriodExpr :: Day -> Text -> Either (ParseError Char MPErr) (Interval, DateSpan)
+parsePeriodExpr refdate s = parsewith (periodexpr refdate <* eof) (T.toLower s)
 
 maybePeriod :: Day -> Text -> Maybe (Interval,DateSpan)
 maybePeriod refdate = either (const Nothing) Just . parsePeriodExpr refdate
@@ -316,13 +326,13 @@ fixSmartDateStr :: Day -> Text -> String
 fixSmartDateStr d s = either
                        (\e->error' $ printf "could not parse date %s %s" (show s) (show e))
                        id
-                       $ (fixSmartDateStrEither d s :: Either (ParseError Char Dec) String)
+                       $ (fixSmartDateStrEither d s :: Either (ParseError Char MPErr) String)
 
 -- | A safe version of fixSmartDateStr.
-fixSmartDateStrEither :: Day -> Text -> Either (ParseError Char Dec) String
+fixSmartDateStrEither :: Day -> Text -> Either (ParseError Char MPErr) String
 fixSmartDateStrEither d = either Left (Right . showDate) . fixSmartDateStrEither' d
 
-fixSmartDateStrEither' :: Day -> Text -> Either (ParseError Char Dec) Day
+fixSmartDateStrEither' :: Day -> Text -> Either (ParseError Char MPErr) Day
 fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
                                Right sd -> Right $ fixSmartDate d sd
                                Left e -> Left e
@@ -447,6 +457,7 @@ thismonth = startofmonth
 prevmonth = startofmonth . addGregorianMonthsClip (-1)
 nextmonth = startofmonth . addGregorianMonthsClip 1
 startofmonth day = fromGregorian y m 1 where (y,m,_) = toGregorian day
+nthdayofmonth d day = fromGregorian y m d where (y,m,_) = toGregorian day
 
 thisquarter = startofquarter
 prevquarter = startofquarter . addGregorianMonthsClip (-3)
@@ -461,18 +472,106 @@ prevyear = startofyear . addGregorianYearsClip (-1)
 nextyear = startofyear . addGregorianYearsClip 1
 startofyear day = fromGregorian y 1 1 where (y,_,_) = toGregorian day
 
-nthdayofmonthcontaining n d | d1 >= d    = d1
-                            | otherwise = d2
-    where d1 = addDays (fromIntegral n-1) s
-          d2 = addDays (fromIntegral n-1) $ nextmonth s
+-- | For given date d find year-long interval that starts on given MM/DD of year
+-- and covers it. 
+--
+-- Examples: lets take 2017-11-22. Year-long intervals covering it that
+-- starts before Nov 22 will start in 2017. However
+-- intervals that start after Nov 23rd should start in 2016:
+-- >>> let wed22nd = parsedate "2017-11-22"          
+-- >>> nthdayofyearcontaining 11 21 wed22nd
+-- 2017-11-21          
+-- >>> nthdayofyearcontaining 11 22 wed22nd
+-- 2017-11-22          
+-- >>> nthdayofyearcontaining 11 23 wed22nd
+-- 2016-11-23          
+-- >>> nthdayofyearcontaining 12 02 wed22nd
+-- 2016-12-02          
+-- >>> nthdayofyearcontaining 12 31 wed22nd
+-- 2016-12-31          
+-- >>> nthdayofyearcontaining 1 1 wed22nd
+-- 2017-01-01          
+nthdayofyearcontaining m n d | mmddOfSameYear <= d = mmddOfSameYear
+                             | otherwise = mmddOfPrevYear
+    where mmddOfSameYear = addDays (fromIntegral n-1) $ applyN (m-1) nextmonth s
+          mmddOfPrevYear = addDays (fromIntegral n-1) $ applyN (m-1) nextmonth $ prevyear s
+          s = startofyear d
+
+-- | For given date d find month-long interval that starts on nth day of month
+-- and covers it. 
+--
+-- Examples: lets take 2017-11-22. Month-long intervals covering it that
+-- start on 1st-22nd of month will start in Nov. However
+-- intervals that start on 23rd-30th of month should start in Oct:
+-- >>> let wed22nd = parsedate "2017-11-22"          
+-- >>> nthdayofmonthcontaining 1 wed22nd
+-- 2017-11-01          
+-- >>> nthdayofmonthcontaining 12 wed22nd
+-- 2017-11-12          
+-- >>> nthdayofmonthcontaining 22 wed22nd
+-- 2017-11-22          
+-- >>> nthdayofmonthcontaining 23 wed22nd
+-- 2017-10-23          
+-- >>> nthdayofmonthcontaining 30 wed22nd
+-- 2017-10-30          
+nthdayofmonthcontaining n d | nthOfSameMonth <= d = nthOfSameMonth
+                            | otherwise = nthOfPrevMonth
+    where nthOfSameMonth = nthdayofmonth n s
+          nthOfPrevMonth = nthdayofmonth n $ prevmonth s
           s = startofmonth d
 
-nthdayofweekcontaining n d | d1 >= d    = d1
-                           | otherwise = d2
-    where d1 = addDays (fromIntegral n-1) s
-          d2 = addDays (fromIntegral n-1) $ nextweek s
+-- | For given date d find week-long interval that starts on nth day of week
+-- and covers it. 
+--
+-- Examples: 2017-11-22 is Wed. Week-long intervals that cover it and
+-- start on Mon, Tue or Wed will start in the same week. However
+-- intervals that start on Thu or Fri should start in prev week:          
+-- >>> let wed22nd = parsedate "2017-11-22"          
+-- >>> nthdayofweekcontaining 1 wed22nd
+-- 2017-11-20          
+-- >>> nthdayofweekcontaining 2 wed22nd
+-- 2017-11-21
+-- >>> nthdayofweekcontaining 3 wed22nd
+-- 2017-11-22          
+-- >>> nthdayofweekcontaining 4 wed22nd
+-- 2017-11-16          
+-- >>> nthdayofweekcontaining 5 wed22nd
+-- 2017-11-17          
+nthdayofweekcontaining n d | nthOfSameWeek <= d = nthOfSameWeek
+                           | otherwise = nthOfPrevWeek
+    where nthOfSameWeek = addDays (fromIntegral n-1) s
+          nthOfPrevWeek = addDays (fromIntegral n-1) $ prevweek s
           s = startofweek d
 
+-- | For given date d find month-long interval that starts on nth weekday of month
+-- and covers it. 
+--
+-- Examples: 2017-11-22 is 3rd Wed of Nov. Month-long intervals that cover it and
+-- start on 1st-4th Wed will start in Nov. However
+-- intervals that start on 4th Thu or Fri or later should start in Oct:          
+-- >>> let wed22nd = parsedate "2017-11-22"          
+-- >>> nthweekdayofmonthcontaining 1 3 wed22nd
+-- 2017-11-01
+-- >>> nthweekdayofmonthcontaining 3 2 wed22nd
+-- 2017-11-21
+-- >>> nthweekdayofmonthcontaining 4 3 wed22nd
+-- 2017-11-22
+-- >>> nthweekdayofmonthcontaining 4 4 wed22nd
+-- 2017-10-26
+-- >>> nthweekdayofmonthcontaining 4 5 wed22nd
+-- 2017-10-27
+nthweekdayofmonthcontaining n wd d | nthWeekdaySameMonth <= d  = nthWeekdaySameMonth
+                                   | otherwise = nthWeekdayPrevMonth
+    where nthWeekdaySameMonth = advancetonthweekday n wd $ startofmonth d
+          nthWeekdayPrevMonth = advancetonthweekday n wd $ prevmonth d
+          
+-- | Advance to nth weekday wd after given start day s
+advancetonthweekday n wd s = addWeeks (n-1) . firstMatch (>=s) . iterate (addWeeks 1) $ firstweekday s
+  where          
+    addWeeks k = addDays (7 * fromIntegral k)
+    firstMatch p = head . dropWhile (not . p)
+    firstweekday = addDays (fromIntegral wd-1) . startofweek
+
 ----------------------------------------------------------------------
 -- parsing
 
@@ -529,11 +628,6 @@ parsedate s =  fromMaybe (error' $ "could not parse date \"" ++ s ++ "\"")
 -- -- 2008-02-29
 -- #endif
 
--- | Parse a time string to a time type using the provided pattern, or
--- return the default.
-_parsetimewith :: ParseTime t => String -> String -> t -> t
-_parsetimewith pat s def = fromMaybe def $ parsetime defaultTimeLocale pat s
-
 {-|
 Parse a date in any of the formats allowed in ledger's period expressions,
 and maybe some others:
@@ -550,14 +644,14 @@ and maybe some others:
 Returns a SmartDate, to be converted to a full date later (see fixSmartDate).
 Assumes any text in the parse stream has been lowercased.
 -}
-smartdate :: Parser SmartDate
+smartdate :: SimpleTextParser SmartDate
 smartdate = do
   -- XXX maybe obscures date errors ? see ledgerdate
   (y,m,d) <- choice' [yyyymmdd, ymd, ym, md, y, d, month, mon, today, yesterday, tomorrow, lastthisnextthing]
   return (y,m,d)
 
 -- | Like smartdate, but there must be nothing other than whitespace after the date.
-smartdateonly :: Parser SmartDate
+smartdateonly :: SimpleTextParser SmartDate
 smartdateonly = do
   d <- smartdate
   many spacenonewline
@@ -579,7 +673,7 @@ failIfInvalidYear s  = unless (validYear s)  $ fail $ "bad year number: " ++ s
 failIfInvalidMonth s = unless (validMonth s) $ fail $ "bad month number: " ++ s
 failIfInvalidDay s   = unless (validDay s)   $ fail $ "bad day number: " ++ s
 
-yyyymmdd :: Parser SmartDate
+yyyymmdd :: SimpleTextParser SmartDate
 yyyymmdd = do
   y <- count 4 digitChar
   m <- count 2 digitChar
@@ -588,7 +682,7 @@ yyyymmdd = do
   failIfInvalidDay d
   return (y,m,d)
 
-ymd :: Parser SmartDate
+ymd :: SimpleTextParser SmartDate
 ymd = do
   y <- some digitChar
   failIfInvalidYear y
@@ -600,7 +694,7 @@ ymd = do
   failIfInvalidDay d
   return $ (y,m,d)
 
-ym :: Parser SmartDate
+ym :: SimpleTextParser SmartDate
 ym = do
   y <- some digitChar
   failIfInvalidYear y
@@ -609,19 +703,19 @@ ym = do
   failIfInvalidMonth m
   return (y,m,"")
 
-y :: Parser SmartDate
+y :: SimpleTextParser SmartDate
 y = do
   y <- some digitChar
   failIfInvalidYear y
   return (y,"","")
 
-d :: Parser SmartDate
+d :: SimpleTextParser SmartDate
 d = do
   d <- some digitChar
   failIfInvalidDay d
   return ("","",d)
 
-md :: Parser SmartDate
+md :: SimpleTextParser SmartDate
 md = do
   m <- some digitChar
   failIfInvalidMonth m
@@ -633,91 +727,115 @@ md = do
 months         = ["january","february","march","april","may","june",
                   "july","august","september","october","november","december"]
 monthabbrevs   = ["jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"]
--- weekdays       = ["monday","tuesday","wednesday","thursday","friday","saturday","sunday"]
--- weekdayabbrevs = ["mon","tue","wed","thu","fri","sat","sun"]
+weekdays       = ["monday","tuesday","wednesday","thursday","friday","saturday","sunday"]
+weekdayabbrevs = ["mon","tue","wed","thu","fri","sat","sun"]
 
-monthIndex s = maybe 0 (+1) $ lowercase s `elemIndex` months
-monIndex s   = maybe 0 (+1) $ lowercase s `elemIndex` monthabbrevs
+monthIndex t = maybe 0 (+1) $ t `elemIndex` months
+monIndex t   = maybe 0 (+1) $ t `elemIndex` monthabbrevs
 
-month :: Parser SmartDate
+month :: SimpleTextParser SmartDate
 month = do
   m <- choice $ map (try . string) months
   let i = monthIndex m
   return ("",show i,"")
 
-mon :: Parser SmartDate
+mon :: SimpleTextParser SmartDate
 mon = do
   m <- choice $ map (try . string) monthabbrevs
   let i = monIndex m
   return ("",show i,"")
 
-today,yesterday,tomorrow :: Parser SmartDate
+weekday :: SimpleTextParser Int
+weekday = do
+  wday <- choice . map string' $ weekdays ++ weekdayabbrevs
+  let i = head . catMaybes $ [wday `elemIndex` weekdays, wday `elemIndex` weekdayabbrevs]
+  return (i+1)
+
+today,yesterday,tomorrow :: SimpleTextParser SmartDate
 today     = string "today"     >> return ("","","today")
 yesterday = string "yesterday" >> return ("","","yesterday")
 tomorrow  = string "tomorrow"  >> return ("","","tomorrow")
 
-lastthisnextthing :: Parser SmartDate
+lastthisnextthing :: SimpleTextParser SmartDate
 lastthisnextthing = do
-  r <- choice [
-        string "last"
-       ,string "this"
-       ,string "next"
+  r <- choice $ map mptext [
+        "last"
+       ,"this"
+       ,"next"
       ]
   many spacenonewline  -- make the space optional for easier scripting
-  p <- choice [
-        string "day"
-       ,string "week"
-       ,string "month"
-       ,string "quarter"
-       ,string "year"
+  p <- choice $ map mptext [
+        "day"
+       ,"week"
+       ,"month"
+       ,"quarter"
+       ,"year"
       ]
 -- XXX support these in fixSmartDate
 --       ++ (map string $ months ++ monthabbrevs ++ weekdays ++ weekdayabbrevs)
 
-  return ("",r,p)
+  return ("", T.unpack r, T.unpack p)
 
 -- |
--- >>> let p = parsewith (periodexpr (parsedate "2008/11/26")) :: T.Text -> Either (ParseError Char Dec) (Interval, DateSpan)
--- >>> p "from aug to oct"
+-- >>> let p = parsePeriodExpr (parsedate "2008/11/26")
+-- >>> p "from Aug to Oct"
 -- Right (NoInterval,DateSpan 2008/08/01-2008/09/30)
 -- >>> p "aug to oct"
 -- Right (NoInterval,DateSpan 2008/08/01-2008/09/30)
--- >>> p "every 3 days in aug"
+-- >>> p "every 3 days in Aug"
 -- Right (Days 3,DateSpan 2008/08)
 -- >>> p "daily from aug"
 -- Right (Days 1,DateSpan 2008/08/01-)
 -- >>> p "every week to 2009"
 -- Right (Weeks 1,DateSpan -2008/12/31)
-periodexpr :: Day -> Parser (Interval, DateSpan)
-periodexpr rdate = choice $ map try [
+-- >>> p "every 2nd day of month"
+-- Right (DayOfMonth 2,DateSpan -)
+-- >>> p "every 2nd day"
+-- Right (DayOfMonth 2,DateSpan -)
+-- >>> p "every 2nd day 2009-"
+-- Right (DayOfMonth 2,DateSpan 2009/01/01-)  
+-- >>> p "every 29th Nov"
+-- Right (DayOfYear 11 29,DateSpan -)
+-- >>> p "every 29th nov -2009"
+-- Right (DayOfYear 11 29,DateSpan -2008/12/31)
+-- >>> p "every nov 29th"
+-- Right (DayOfYear 11 29,DateSpan -)
+-- >>> p "every Nov 29th 2009-"
+-- Right (DayOfYear 11 29,DateSpan 2009/01/01-)
+-- >>> p "every 11/29 from 2009"
+-- Right (DayOfYear 11 29,DateSpan 2009/01/01-)
+-- >>> p "every 2nd Thursday of month to 2009"
+-- Right (WeekdayOfMonth 2 4,DateSpan -2008/12/31)
+-- >>> p "every 1st monday of month to 2009"
+-- Right (WeekdayOfMonth 1 1,DateSpan -2008/12/31)
+-- >>> p "every tue"
+-- Right (DayOfWeek 2,DateSpan -)
+-- >>> p "every 2nd day of week"
+-- Right (DayOfWeek 2,DateSpan -)
+-- >>> p "every 2nd day of month"
+-- Right (DayOfMonth 2,DateSpan -)
+-- >>> p "every 2nd day"
+-- Right (DayOfMonth 2,DateSpan -)
+-- >>> p "every 2nd day 2009-"
+-- Right (DayOfMonth 2,DateSpan 2009/01/01-)
+-- >>> p "every 2nd day of month 2009-"
+-- Right (DayOfMonth 2,DateSpan 2009/01/01-)
+periodexpr :: Day -> SimpleTextParser (Interval, DateSpan)
+periodexpr rdate = surroundedBy (many spacenonewline) . choice $ map try [
                     intervalanddateperiodexpr rdate,
-                    intervalperiodexpr,
-                    dateperiodexpr rdate,
-                    (return (NoInterval,DateSpan Nothing Nothing))
+                    (,) NoInterval <$> periodexprdatespan rdate
                    ]
 
-intervalanddateperiodexpr :: Day -> Parser (Interval, DateSpan)
+intervalanddateperiodexpr :: Day -> SimpleTextParser (Interval, DateSpan)
 intervalanddateperiodexpr rdate = do
-  many spacenonewline
   i <- reportinginterval
-  many spacenonewline
-  s <- periodexprdatespan rdate
+  s <- option def . try $ do
+      many spacenonewline
+      periodexprdatespan rdate
   return (i,s)
 
-intervalperiodexpr :: Parser (Interval, DateSpan)
-intervalperiodexpr = do
-  many spacenonewline
-  i <- reportinginterval
-  return (i, DateSpan Nothing Nothing)
-
-dateperiodexpr :: Day -> Parser (Interval, DateSpan)
-dateperiodexpr rdate = do
-  many spacenonewline
-  s <- periodexprdatespan rdate
-  return (NoInterval, s)
-
 -- Parse a reporting interval.
-reportinginterval :: Parser Interval
+reportinginterval :: SimpleTextParser Interval
 reportinginterval = choice' [
                        tryinterval "day"     "daily"     Days,
                        tryinterval "week"    "weekly"    Weeks,
@@ -730,52 +848,76 @@ reportinginterval = choice' [
                           return $ Months 2,
                        do string "every"
                           many spacenonewline
-                          n <- fmap read $ some digitChar
-                          thsuffix
+                          n <- nth
                           many spacenonewline
                           string "day"
-                          many spacenonewline
-                          string "of"
-                          many spacenonewline
-                          string "week"
+                          of_ "week"
                           return $ DayOfWeek n,
                        do string "every"
                           many spacenonewline
-                          n <- fmap read $ some digitChar
-                          thsuffix
+                          n <- weekday
+                          return $ DayOfWeek n,
+                       do string "every"
+                          many spacenonewline
+                          n <- nth
                           many spacenonewline
                           string "day"
-                          optional $ do
-                            many spacenonewline
-                            string "of"
-                            many spacenonewline
-                            string "month"
-                          return $ DayOfMonth n
+                          optOf_ "month"
+                          return $ DayOfMonth n,
+                       do string "every"
+                          let mnth = choice' [month, mon] >>= \(_,m,_) -> return (read m)
+                          d_o_y <- makePermParser $ DayOfYear <$$> try (many spacenonewline *> mnth) <||> try (many spacenonewline *> nth)
+                          optOf_ "year"
+                          return d_o_y,
+                       do string "every"
+                          many spacenonewline
+                          ("",m,d) <- md
+                          optOf_ "year"
+                          return $ DayOfYear (read m) (read d),
+                       do string "every"
+                          many spacenonewline
+                          n <- nth
+                          many spacenonewline
+                          wd <- weekday
+                          optOf_ "month"
+                          return $ WeekdayOfMonth n wd
                     ]
     where
-
-      thsuffix = choice' $ map string ["st","nd","rd","th"]
+      of_ period = do
+        many spacenonewline
+        string "of"
+        many spacenonewline
+        string period
+        
+      optOf_ period = optional $ try $ of_ period
+      
+      nth = do n <- some digitChar
+               choice' $ map string ["st","nd","rd","th"]
+               return $ read n
 
       -- Parse any of several variants of a basic interval, eg "daily", "every day", "every N days".
-      tryinterval :: String -> String -> (Int -> Interval) -> Parser Interval
+      tryinterval :: String -> String -> (Int -> Interval) -> SimpleTextParser Interval
       tryinterval singular compact intcons =
-          choice' [
-           do string compact
-              return $ intcons 1,
-           do string "every"
-              many spacenonewline
-              string singular
-              return $ intcons 1,
-           do string "every"
-              many spacenonewline
-              n <- fmap read $ some digitChar
-              many spacenonewline
-              string plural
-              return $ intcons n
-           ]
-          where plural = singular ++ "s"
+        choice' [
+          do mptext compact'
+             return $ intcons 1,
+          do mptext "every"
+             many spacenonewline
+             mptext singular'
+             return $ intcons 1,
+          do mptext "every"
+             many spacenonewline
+             n <- fmap read $ some digitChar
+             many spacenonewline
+             mptext plural'
+             return $ intcons n
+          ]
+        where
+          compact'  = T.pack compact
+          singular' = T.pack singular
+          plural'   = T.pack $ singular ++ "s"
 
-periodexprdatespan :: Day -> Parser DateSpan
+periodexprdatespan :: Day -> SimpleTextParser DateSpan
 periodexprdatespan rdate = choice $ map try [
                             doubledatespan rdate,
                             fromdatespan rdate,
@@ -783,7 +925,7 @@ periodexprdatespan rdate = choice $ map try [
                             justdatespan rdate
                            ]
 
-doubledatespan :: Day -> Parser DateSpan
+doubledatespan :: Day -> SimpleTextParser DateSpan
 doubledatespan rdate = do
   optional (string "from" >> many spacenonewline)
   b <- smartdate
@@ -792,7 +934,7 @@ doubledatespan rdate = do
   e <- smartdate
   return $ DateSpan (Just $ fixSmartDate rdate b) (Just $ fixSmartDate rdate e)
 
-fromdatespan :: Day -> Parser DateSpan
+fromdatespan :: Day -> SimpleTextParser DateSpan
 fromdatespan rdate = do
   b <- choice [
     do
@@ -806,13 +948,13 @@ fromdatespan rdate = do
     ]
   return $ DateSpan (Just $ fixSmartDate rdate b) Nothing
 
-todatespan :: Day -> Parser DateSpan
+todatespan :: Day -> SimpleTextParser DateSpan
 todatespan rdate = do
   choice [string "to", string "-"] >> many spacenonewline
   e <- smartdate
   return $ DateSpan Nothing (Just $ fixSmartDate rdate e)
 
-justdatespan :: Day -> Parser DateSpan
+justdatespan :: Day -> SimpleTextParser DateSpan
 justdatespan rdate = do
   optional (string "in" >> many spacenonewline)
   d <- smartdate
