@@ -1,5 +1,7 @@
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE StandaloneDeriving, OverloadedStrings #-}
+{-# LANGUAGE CPP #-}
+
 {-|
 
 A 'Journal' is a set of transactions, plus optional related data.  This is
@@ -20,6 +22,7 @@ module Hledger.Data.Journal (
   commodityStylesFromAmounts,
   journalConvertAmountsToCost,
   journalFinalise,
+  journalPivot,
   -- * Filtering
   filterJournalTransactions,
   filterJournalPostings,
@@ -28,8 +31,12 @@ module Hledger.Data.Journal (
   filterTransactionPostings,
   filterPostingAmount,
   -- * Querying
-  journalAccountNames,
   journalAccountNamesUsed,
+  journalAccountNamesImplied,
+  journalAccountNamesDeclared,
+  journalAccountNamesDeclaredOrUsed,
+  journalAccountNamesDeclaredOrImplied,
+  journalAccountNames,
   -- journalAmountAndPriceCommodities,
   journalAmounts,
   overJournalAmounts,
@@ -74,10 +81,12 @@ import Data.Array.ST
 import Data.Functor.Identity (Identity(..))
 import qualified Data.HashTable.ST.Cuckoo as HT
 import Data.List
+import Data.List.Extra (groupSort)
 -- import Data.Map (findWithDefault)
 import Data.Maybe
 import Data.Monoid
 import Data.Ord
+import qualified Data.Semigroup as Sem
 import Data.Text (Text)
 import qualified Data.Text as T
 import Safe (headMay, headDef)
@@ -153,9 +162,8 @@ instance Show Journal where
 -- CHILD <> PARENT. A parsed journal's data is in reverse order, so
 -- this gives what we want.
 --
-instance Monoid Journal where
-  mempty = nulljournal
-  mappend j1 j2 = Journal {
+instance Sem.Semigroup Journal where
+  j1 <> j2 = Journal {
      jparsedefaultyear          = jparsedefaultyear          j2
     ,jparsedefaultcommodity     = jparsedefaultcommodity     j2
     ,jparseparentaccounts       = jparseparentaccounts       j2
@@ -174,6 +182,13 @@ instance Monoid Journal where
     ,jlastreadtime              = max (jlastreadtime j1) (jlastreadtime j2)
     }
 
+instance Monoid Journal where
+  mempty = nulljournal
+#if !(MIN_VERSION_base(4,11,0))
+  -- This is redundant starting with base-4.11 / GHC 8.4.
+  mappend = (Sem.<>)
+#endif
+
 nulljournal :: Journal
 nulljournal = Journal {
    jparsedefaultyear          = Nothing
@@ -237,13 +252,32 @@ journalDescriptions = nub . sort . map tdescription . jtxns
 journalPostings :: Journal -> [Posting]
 journalPostings = concatMap tpostings . jtxns
 
--- | Unique account names posted to in this journal.
+-- | Sorted unique account names posted to by this journal's transactions.
 journalAccountNamesUsed :: Journal -> [AccountName]
-journalAccountNamesUsed = sort . accountNamesFromPostings . journalPostings
+journalAccountNamesUsed = accountNamesFromPostings . journalPostings
 
--- | Unique account names in this journal, including parent accounts containing no postings.
+-- | Sorted unique account names implied by this journal's transactions - 
+-- accounts posted to and all their implied parent accounts.
+journalAccountNamesImplied :: Journal -> [AccountName]
+journalAccountNamesImplied = expandAccountNames . journalAccountNamesUsed
+
+-- | Sorted unique account names declared by account directives in this journal.
+journalAccountNamesDeclared :: Journal -> [AccountName]
+journalAccountNamesDeclared = nub . sort . jaccounts
+
+-- | Sorted unique account names declared by account directives or posted to
+-- by transactions in this journal.
+journalAccountNamesDeclaredOrUsed :: Journal -> [AccountName]
+journalAccountNamesDeclaredOrUsed j = nub $ sort $ journalAccountNamesDeclared j ++ journalAccountNamesUsed j
+
+-- | Sorted unique account names declared by account directives, or posted to
+-- or implied as parents by transactions in this journal.
+journalAccountNamesDeclaredOrImplied :: Journal -> [AccountName]
+journalAccountNamesDeclaredOrImplied j = nub $ sort $ journalAccountNamesDeclared j ++ journalAccountNamesImplied j
+
+-- | Convenience/compatibility alias for journalAccountNamesDeclaredOrImplied.
 journalAccountNames :: Journal -> [AccountName]
-journalAccountNames = sort . expandAccountNames . journalAccountNamesUsed
+journalAccountNames = journalAccountNamesDeclaredOrImplied 
 
 journalAccountNameTree :: Journal -> Tree AccountName
 journalAccountNameTree = accountNameTreeFrom . journalAccountNames
@@ -293,9 +327,9 @@ journalEquityAccountQuery _ = Acct "^equity(:|$)"
 -- | A query for Cash (-equivalent) accounts in this journal (ie,
 -- accounts which appear on the cashflow statement.)  This is currently
 -- hard-coded to be all the Asset accounts except for those containing the
--- case-insensitive regex @(receivable|A/R)@.
+-- case-insensitive regex @(receivable|:A/R|:fixed)@.
 journalCashAccountQuery  :: Journal -> Query
-journalCashAccountQuery j = And [journalAssetAccountQuery j, Not $ Acct "(receivable|A/R)"]
+journalCashAccountQuery j = And [journalAssetAccountQuery j, Not $ Acct "(receivable|:A/R|:fixed)"]
 
 -- Various kinds of filtering on journals. We do it differently depending
 -- on the command.
@@ -346,7 +380,7 @@ filterJournalTransactions FilterSpec{datespan=datespan
                                     ,depth=depth
                                     ,fMetadata=md
                                     } =
-    filterJournalTransactionsByClearedStatus cleared .
+    filterJournalTransactionsByStatus cleared .
     filterJournalPostingsByDepth depth .
     filterJournalTransactionsByAccount apats .
     filterJournalTransactionsByMetadata md .
@@ -366,7 +400,7 @@ filterJournalPostings FilterSpec{datespan=datespan
                                 ,fMetadata=md
                                 } =
     filterJournalPostingsByRealness real .
-    filterJournalPostingsByClearedStatus cleared .
+    filterJournalPostingsByStatus cleared .
     filterJournalPostingsByEmpty empty .
     filterJournalPostingsByDepth depth .
     filterJournalPostingsByAccount apats .
@@ -393,16 +427,16 @@ filterJournalTransactionsByDate (DateSpan begin end) j@Journal{jtxns=ts} = j{jtx
 
 -- | Keep only transactions which have the requested cleared/uncleared
 -- status, if there is one.
-filterJournalTransactionsByClearedStatus :: Maybe Bool -> Journal -> Journal
-filterJournalTransactionsByClearedStatus Nothing j = j
-filterJournalTransactionsByClearedStatus (Just val) j@Journal{jtxns=ts} = j{jtxns=filter match ts}
+filterJournalTransactionsByStatus :: Maybe Bool -> Journal -> Journal
+filterJournalTransactionsByStatus Nothing j = j
+filterJournalTransactionsByStatus (Just val) j@Journal{jtxns=ts} = j{jtxns=filter match ts}
     where match = (==val).tstatus
 
 -- | Keep only postings which have the requested cleared/uncleared status,
 -- if there is one.
-filterJournalPostingsByClearedStatus :: Maybe Bool -> Journal -> Journal
-filterJournalPostingsByClearedStatus Nothing j = j
-filterJournalPostingsByClearedStatus (Just c) j@Journal{jtxns=ts} = j{jtxns=map filterpostings ts}
+filterJournalPostingsByStatus :: Maybe Bool -> Journal -> Journal
+filterJournalPostingsByStatus Nothing j = j
+filterJournalPostingsByStatus (Just c) j@Journal{jtxns=ts} = j{jtxns=map filterpostings ts}
     where filterpostings t@Transaction{tpostings=ps} = t{tpostings=filter ((==c) . postingCleared) ps}
 
 -- | Strip out any virtual postings, if the flag is true, otherwise do
@@ -512,7 +546,7 @@ journalCheckBalanceAssertions j =
 -- | Check a posting's balance assertion and return an error if it
 -- fails.
 checkBalanceAssertion :: Posting -> MixedAmount -> Either String ()
-checkBalanceAssertion p@Posting{ pbalanceassertion = Just ass} amt
+checkBalanceAssertion p@Posting{ pbalanceassertion = Just (ass,_)} amt
   | isReallyZeroAmount diff = Right ()
   | True    = Left err
     where assertedcomm = acommodity ass
@@ -534,7 +568,8 @@ checkBalanceAssertion p@Posting{ pbalanceassertion = Just ass} amt
             (case ptransaction p of
                Nothing -> ":" -- shouldn't happen
                Just t ->  printf " in %s:\nin transaction:\n%s"
-                          (showGenericSourcePos $ tsourcepos t) (chomp $ show t) :: String)
+                          (showGenericSourcePos pos) (chomp $ show t) :: String
+                            where pos = snd $ fromJust $ pbalanceassertion p)
             (showPostingLine p)
             (showDate $ postingDate p)
             (T.unpack $ paccount p) -- XXX pack
@@ -561,11 +596,13 @@ type CurrentBalancesModifier s = R.ReaderT (Env s) (ExceptT String (ST s))
 -- depends on display precision. Reports only the first error encountered.
 journalBalanceTransactions :: Bool -> Journal -> Either String Journal
 journalBalanceTransactions assrt j =
-  runST $ journalBalanceTransactionsST assrt (journalNumberTransactions j)
-  (newArray_ (1, genericLength $ jtxns j)
-   :: forall s. ST s (STArray s Integer Transaction))
-  (\arr tx -> writeArray arr (tindex tx) tx)
-  $ fmap (\txns -> j{ jtxns = txns}) . getElems
+  runST $ 
+    journalBalanceTransactionsST 
+      assrt -- check balance assertions also ?
+      (journalNumberTransactions j) -- journal to process
+      (newArray_ (1, genericLength $ jtxns j) :: forall s. ST s (STArray s Integer Transaction)) -- initialise state
+      (\arr tx -> writeArray arr (tindex tx) tx)    -- update state
+      (fmap (\txns -> j{ jtxns = txns}) . getElems) -- summarise state
 
 
 -- | Generalization used in the definition of
@@ -660,7 +697,7 @@ checkInferAndRegisterAmounts (Right oldTx) = do
   where
     inferFromAssignment :: Posting -> CurrentBalancesModifier s Posting
     inferFromAssignment p = maybe (return p)
-      (fmap (\a -> p { pamount = a, porigin = Just $ originalPosting p }) . setBalance (paccount p))
+      (fmap (\a -> p { pamount = a, porigin = Just $ originalPosting p }) . setBalance (paccount p) . fst)
       $ pbalanceassertion p
 
 -- | Adds a posting's amonut to the posting's account balance and
@@ -727,8 +764,11 @@ journalApplyCommodityStyles j@Journal{jtxns=ts, jmarketprices=mps} = j''
 -- from the posting amounts (or in some cases, price amounts) in this
 -- commodity if any, otherwise the default style.
 journalCommodityStyle :: Journal -> CommoditySymbol -> AmountStyle
-journalCommodityStyle j c =
-  headDef amountstyle{asprecision=2} $
+journalCommodityStyle j = fromMaybe amountstyle{asprecision=2} . journalCommodityStyleLookup j
+
+journalCommodityStyleLookup :: Journal -> CommoditySymbol -> Maybe AmountStyle
+journalCommodityStyleLookup j c =
+  listToMaybe $
   catMaybes [
      M.lookup c (jcommodities j) >>= cformat
     ,M.lookup c $ jinferredcommodities j
@@ -748,8 +788,7 @@ journalInferCommodityStyles j =
 commodityStylesFromAmounts :: [Amount] -> M.Map CommoditySymbol AmountStyle
 commodityStylesFromAmounts amts = M.fromList commstyles
   where
-    samecomm = \a1 a2 -> acommodity a1 == acommodity a2
-    commamts = [(acommodity $ head as, as) | as <- groupBy samecomm $ sortBy (comparing acommodity) amts]
+    commamts = groupSort [(acommodity as, as) | as <- amts]
     commstyles = [(c, canonicalStyleFrom $ map astyle as) | (c,as) <- commamts]
 
 -- | Given an ordered list of amount styles, choose a canonical style.
@@ -798,7 +837,10 @@ journalConvertAmountsToCost j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
       fixtransaction t@Transaction{tpostings=ps} = t{tpostings=map fixposting ps}
       fixposting p@Posting{pamount=a} = p{pamount=fixmixedamount a}
       fixmixedamount (Mixed as) = Mixed $ map fixamount as
-      fixamount = canonicaliseAmount (jinferredcommodities j) . costOfAmount
+      fixamount = applyJournalStyle . costOfAmount
+      applyJournalStyle a
+          | Just s <- journalCommodityStyleLookup j (acommodity a) = a{astyle=s}
+          | otherwise = a
 
 -- -- | Get this journal's unique, display-preference-canonicalised commodities, by symbol.
 -- journalCanonicalCommodities :: Journal -> M.Map String CommoditySymbol
@@ -883,6 +925,32 @@ test_journalDateSpan = do
                             ]}
 -- #endif
 
+-- | Apply the pivot transformation to all postings in a journal,
+-- replacing their account name by their value for the given field or tag.
+journalPivot :: Text -> Journal -> Journal
+journalPivot fieldortagname j = j{jtxns = map (transactionPivot fieldortagname) . jtxns $ j}
+
+-- | Replace this transaction's postings' account names with the value
+-- of the given field or tag, if any.
+transactionPivot :: Text -> Transaction -> Transaction         
+transactionPivot fieldortagname t = t{tpostings = map (postingPivot fieldortagname) . tpostings $ t}
+
+-- | Replace this posting's account name with the value
+-- of the given field or tag, if any, otherwise the empty string.
+postingPivot :: Text -> Posting -> Posting         
+postingPivot fieldortagname p = p{paccount = pivotedacct, porigin = Just $ originalPosting p}
+  where
+    pivotedacct
+      | Just t <- ptransaction p, fieldortagname == "code"        = tcode t  
+      | Just t <- ptransaction p, fieldortagname == "description" = tdescription t  
+      | Just t <- ptransaction p, fieldortagname == "payee"       = transactionPayee t  
+      | Just t <- ptransaction p, fieldortagname == "note"        = transactionNote t  
+      | Just (_, value) <- postingFindTag fieldortagname p        = value
+      | otherwise                                                 = ""
+
+postingFindTag :: TagName -> Posting -> Maybe (TagName, TagValue)         
+postingFindTag tagname p = find ((tagname==) . fst) $ postingAllTags p
+
 -- Misc helpers
 
 -- | Check if a set of hledger account/description filter patterns matches the
@@ -927,6 +995,10 @@ abspat pat = if isnegativepat pat then drop (length negateprefix) pat else pat
 --     expenses:supplies  $1
 --     assets:cash
 --
+-- 2008/10/01 take a loan
+--     assets:bank:checking $1
+--     liabilities:debts    $-1
+--
 -- 2008/12/31 * pay off
 --     liabilities:debts  $1
 --     assets:bank:checking
@@ -939,7 +1011,7 @@ Right samplejournal = journalBalanceTransactions False $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/01/01",
              tdate2=Nothing,
-             tstatus=Uncleared,
+             tstatus=Unmarked,
              tcode="",
              tdescription="income",
              tcomment="",
@@ -956,7 +1028,7 @@ Right samplejournal = journalBalanceTransactions False $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/01",
              tdate2=Nothing,
-             tstatus=Uncleared,
+             tstatus=Unmarked,
              tcode="",
              tdescription="gift",
              tcomment="",
@@ -973,7 +1045,7 @@ Right samplejournal = journalBalanceTransactions False $
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/06/02",
              tdate2=Nothing,
-             tstatus=Uncleared,
+             tstatus=Unmarked,
              tcode="",
              tdescription="save",
              tcomment="",
@@ -1001,13 +1073,29 @@ Right samplejournal = journalBalanceTransactions False $
                        ],
              tpreceding_comment_lines=""
            }
+          ,
+           txnTieKnot $ Transaction {
+             tindex=0,
+             tsourcepos=nullsourcepos,
+             tdate=parsedate "2008/10/01",
+             tdate2=Nothing,
+             tstatus=Unmarked,
+             tcode="",
+             tdescription="take a loan",
+             tcomment="",
+             ttags=[],
+             tpostings=["assets:bank:checking" `post` usd 1
+                       ,"liabilities:debts" `post` usd (-1)
+                       ],
+             tpreceding_comment_lines=""
+           }
           ,
            txnTieKnot $ Transaction {
              tindex=0,
              tsourcepos=nullsourcepos,
              tdate=parsedate "2008/12/31",
              tdate2=Nothing,
-             tstatus=Uncleared,
+             tstatus=Unmarked,
              tcode="",
              tdescription="pay off",
              tcomment="",
