@@ -25,7 +25,6 @@ module Hledger.Data.Posting (
   hasAmount,
   postingAllTags,
   transactionAllTags,
-  postingAllImplicitTags,
   relatedPostings,
   removePrices,
   -- * date operations
@@ -81,7 +80,7 @@ nullposting, posting :: Posting
 nullposting = Posting
                 {pdate=Nothing
                 ,pdate2=Nothing
-                ,pstatus=Uncleared
+                ,pstatus=Unmarked
                 ,paccount=""
                 ,pamount=nullmixedamt
                 ,pcomment=""
@@ -96,6 +95,7 @@ posting = nullposting
 post :: AccountName -> Amount -> Posting
 post acct amt = posting {paccount=acct, pamount=Mixed [amt]}
 
+-- Get the original posting, if any.
 originalPosting :: Posting -> Posting
 originalPosting p = fromMaybe p $ porigin p
 
@@ -132,8 +132,9 @@ hasAmount = (/= missingmixedamt) . pamount
 isAssignment :: Posting -> Bool
 isAssignment p = not (hasAmount p) && isJust (pbalanceassertion p)
 
+-- | Sorted unique account names referenced by these postings.
 accountNamesFromPostings :: [Posting] -> [AccountName]
-accountNamesFromPostings = nub . map paccount
+accountNamesFromPostings = nub . sort . map paccount
 
 sumPostings :: [Posting] -> MixedAmount
 sumPostings = sumStrict . map pamount
@@ -163,43 +164,33 @@ postingDate2 p = headDef nulldate $ catMaybes dates
                 ,maybe Nothing (Just . tdate) $ ptransaction p
                 ]
 
--- | Get a posting's cleared status: cleared or pending if those are
--- explicitly set, otherwise the cleared status of its parent
--- transaction, or uncleared if there is no parent transaction. (Note
--- Uncleared's ambiguity, it can mean "uncleared" or "don't know".
-postingStatus :: Posting -> ClearedStatus
+-- | Get a posting's status. This is cleared or pending if those are
+-- explicitly set on the posting, otherwise the status of its parent
+-- transaction, or unmarked if there is no parent transaction. (Note
+-- the ambiguity, unmarked can mean "posting and transaction are both 
+-- unmarked" or "posting is unmarked and don't know about the transaction".
+postingStatus :: Posting -> Status
 postingStatus Posting{pstatus=s, ptransaction=mt}
-  | s == Uncleared = case mt of Just t  -> tstatus t
-                                Nothing -> Uncleared
+  | s == Unmarked = case mt of Just t  -> tstatus t
+                               Nothing -> Unmarked
   | otherwise = s
 
--- | Implicit tags for this transaction.
-transactionImplicitTags :: Transaction -> [Tag]
-transactionImplicitTags t = filter (not . T.null . snd) [("code", tcode t)
-                                                        ,("description", tdescription t)
-                                                        ,("payee", transactionPayee t)
-                                                        ,("note", transactionNote t)
-                                                        ]
-
 transactionPayee :: Transaction -> Text
 transactionPayee = fst . payeeAndNoteFromDescription . tdescription
 
 transactionNote :: Transaction -> Text
-transactionNote = fst . payeeAndNoteFromDescription . tdescription
+transactionNote = snd . payeeAndNoteFromDescription . tdescription
 
 -- | Parse a transaction's description into payee and note (aka narration) fields,
 -- assuming a convention of separating these with | (like Beancount).
 -- Ie, everything up to the first | is the payee, everything after it is the note.
 -- When there's no |, payee == note == description.
 payeeAndNoteFromDescription :: Text -> (Text,Text)
-payeeAndNoteFromDescription t = (textstrip p, textstrip $ T.tail n)
+payeeAndNoteFromDescription t
+  | T.null n = (t, t)
+  | otherwise = (textstrip p, textstrip $ T.drop 1 n)
   where
-    (p,n) = T.breakOn "|" t
-
--- | Tags for this posting including implicit and any inherited from its parent transaction.
-postingAllImplicitTags :: Posting -> [Tag]
-postingAllImplicitTags p = ptags p ++ maybe [] transactionTags (ptransaction p)
-    where transactionTags t = ttags t ++ transactionImplicitTags t
+    (p, n) = T.span (/= '|') t
 
 -- | Tags for this posting including any inherited from its parent transaction.
 postingAllTags :: Posting -> [Tag]
