@@ -43,6 +43,8 @@ data WhichDate = PrimaryDate | SecondaryDate deriving (Eq,Show)
 
 data DateSpan = DateSpan (Maybe Day) (Maybe Day) deriving (Eq,Ord,Data,Generic,Typeable)
 
+instance Default DateSpan where def = DateSpan Nothing Nothing
+
 instance NFData DateSpan
 
 -- synonyms for various date-related scalars
@@ -89,7 +91,9 @@ data Interval =
   | Quarters Int
   | Years Int
   | DayOfMonth Int
+  | WeekdayOfMonth Int Int
   | DayOfWeek Int
+  | DayOfYear Int Int -- Month, Day
   -- WeekOfYear Int
   -- MonthOfYear Int
   -- QuarterOfYear Int
@@ -158,10 +162,11 @@ data Commodity = Commodity {
 instance NFData Commodity
 
 data Amount = Amount {
-      acommodity :: CommoditySymbol,
-      aquantity  :: Quantity,
-      aprice     :: Price,           -- ^ the (fixed) price for this amount, if any
-      astyle     :: AmountStyle
+      acommodity  :: CommoditySymbol,
+      aquantity   :: Quantity,
+      aprice      :: Price,           -- ^ the (fixed) price for this amount, if any
+      astyle      :: AmountStyle,
+      amultiplier :: Bool             -- ^ amount is a multipier for AutoTransactions
     } deriving (Eq,Ord,Typeable,Data,Generic)
 
 instance NFData Amount
@@ -179,26 +184,30 @@ type TagName = Text
 type TagValue = Text
 type Tag = (TagName, TagValue)  -- ^ A tag name and (possibly empty) value.
 
-data ClearedStatus = Uncleared | Pending | Cleared
-  deriving (Eq,Ord,Typeable,Data,Generic)
+-- | The status of a transaction or posting, recorded with a status mark
+-- (nothing, !, or *). What these mean is ultimately user defined.
+data Status = Unmarked | Pending | Cleared
+  deriving (Eq,Ord,Bounded,Enum,Typeable,Data,Generic)
 
-instance NFData ClearedStatus
+instance NFData Status
 
-instance Show ClearedStatus where -- custom show.. bad idea.. don't do it..
-  show Uncleared = ""
+instance Show Status where -- custom show.. bad idea.. don't do it..
+  show Unmarked = ""
   show Pending   = "!"
   show Cleared   = "*"
 
+type BalanceAssertion = Maybe (Amount, GenericSourcePos)
+
 data Posting = Posting {
       pdate             :: Maybe Day,         -- ^ this posting's date, if different from the transaction's
       pdate2            :: Maybe Day,         -- ^ this posting's secondary date, if different from the transaction's
-      pstatus           :: ClearedStatus,
+      pstatus           :: Status,
       paccount          :: AccountName,
       pamount           :: MixedAmount,
       pcomment          :: Text,              -- ^ this posting's comment lines, as a single non-indented multi-line string
       ptype             :: PostingType,
       ptags             :: [Tag],             -- ^ tag names and values, extracted from the comment
-      pbalanceassertion :: Maybe Amount,      -- ^ optional: the expected balance in this commodity in the account after this posting
+      pbalanceassertion :: BalanceAssertion,  -- ^ optional: the expected balance in this commodity in the account after this posting
       ptransaction      :: Maybe Transaction, -- ^ this posting's parent transaction (co-recursive types).
                                               -- Tying this knot gets tedious, Maybe makes it easier/optional.
       porigin           :: Maybe Posting      -- ^ original posting if this one is result of any transformations (one level only)
@@ -224,7 +233,7 @@ data Transaction = Transaction {
       tsourcepos               :: GenericSourcePos,
       tdate                    :: Day,
       tdate2                   :: Maybe Day,
-      tstatus                  :: ClearedStatus,
+      tstatus                  :: Status,
       tcode                    :: Text,
       tdescription             :: Text,
       tcomment                 :: Text,      -- ^ this transaction's comment lines, as a single non-indented multi-line string
@@ -353,6 +362,15 @@ data Account = Account {
   aboring                   :: Bool           -- ^ used in the accounts report to label elidable parents
   } deriving (Typeable, Data, Generic)
 
+-- | Whether an account's balance is normally a positive number (in accounting terms,
+-- normally a debit balance), as for asset and expense accounts, or a negative number
+-- (in accounting terms, normally a credit balance), as for liability, equity and 
+-- income accounts. Cf https://en.wikipedia.org/wiki/Normal_balance .
+data NormalBalance = 
+    NormalPositive -- ^ normally debit - assets, expenses...
+  | NormalNegative -- ^ normally credit - liabilities, equity, income...
+  deriving (Show, Data, Eq) 
+
 -- | A Ledger has the journal it derives from, and the accounts
 -- derived from that. Accounts are accessible both list-wise and
 -- tree-wise, since each one knows its parent and subs; the first
