@@ -14,6 +14,7 @@ module Hledger.Read (
   PrefixedFilePath,
   defaultJournal,
   defaultJournalPath,
+  readJournalFilesWithOpts,
   readJournalFiles,
   readJournalFile,
   requireJournalFileExists,
@@ -41,18 +42,20 @@ import qualified Control.Exception as C
 import Control.Monad.Except
 import Data.List
 import Data.Maybe
+import Data.Ord
 import Data.Text (Text)
 import qualified Data.Text as T
+import Data.Time (Day)
 import Safe
 import System.Directory (doesFileExist, getHomeDirectory)
 import System.Environment (getEnv)
 import System.Exit (exitFailure)
-import System.FilePath ((</>), takeExtension)
-import System.IO (stderr)
+import System.FilePath
+import System.IO
 import Test.HUnit
 import Text.Printf
 
-import Hledger.Data.Dates (getCurrentDay)
+import Hledger.Data.Dates (getCurrentDay, parsedate, showDate)
 import Hledger.Data.Types
 import Hledger.Read.Common
 import qualified Hledger.Read.JournalReader   as JournalReader
@@ -62,7 +65,6 @@ import qualified Hledger.Read.TimeclockReader as TimeclockReader
 import qualified Hledger.Read.CsvReader       as CsvReader
 import Hledger.Utils
 import Prelude hiding (getContents, writeFile)
-import Hledger.Utils.UTF8IOCompat (writeFile)
 
 
 journalEnvVar           = "LEDGER_FILE"
@@ -258,6 +260,111 @@ tryReaders readers mrulesfile assrt path t = firstSuccessOrFirstError [] readers
     path' = fromMaybe "(string)" path
 
 
+--- New versions of readJournal* with easier arguments, and support for --new.
+
+readJournalFilesWithOpts :: InputOpts -> [FilePath] -> IO (Either String Journal)
+readJournalFilesWithOpts iopts =
+  (right mconcat1 . sequence <$>) . mapM (readJournalFileWithOpts iopts)
+  where
+    mconcat1 :: Monoid t => [t] -> t
+    mconcat1 [] = mempty
+    mconcat1 x  = foldr1 mappend x
+
+readJournalFileWithOpts :: InputOpts -> PrefixedFilePath -> IO (Either String Journal)
+readJournalFileWithOpts iopts prefixedfile = do
+  let 
+    (mfmt, f) = splitReaderPrefix prefixedfile
+    iopts' = iopts{mformat_=firstJust [mfmt, mformat_ iopts]}
+  requireJournalFileExists f
+  t <- readFileOrStdinAnyLineEnding f
+  ej <- readJournalWithOpts iopts' (Just f) t
+  case ej of
+    Left e  -> return $ Left e
+    Right j | new_ iopts -> do
+      ds <- previousLatestDates f
+      let (newj, newds) = journalFilterSinceLatestDates ds j
+      when (new_save_ iopts && not (null newds)) $ saveLatestDates newds f
+      return $ Right newj
+    Right j -> return $ Right j
+
+-- A "LatestDates" is zero or more copies of the same date,
+-- representing the latest transaction date read from a file,
+-- and how many transactions there were on that date.
+type LatestDates = [Day]
+
+-- | Get all instances of the latest date in an unsorted list of dates.
+-- Ie, if the latest date appears once, return it in a one-element list,
+-- if it appears three times (anywhere), return three of it.
+latestDates :: [Day] -> LatestDates
+latestDates = headDef [] . take 1 . group . reverse . sort
+
+-- | Remember that these transaction dates were the latest seen when
+-- reading this journal file.
+saveLatestDates :: LatestDates -> FilePath -> IO () 
+saveLatestDates dates f = writeFile (latestDatesFileFor f) $ unlines $ map showDate dates
+
+-- | What were the latest transaction dates seen the last time this 
+-- journal file was read ? If there were multiple transactions on the
+-- latest date, that number of dates is returned, otherwise just one.
+-- Or none if no transactions were read, or if latest dates info is not 
+-- available for this file.
+previousLatestDates :: FilePath -> IO LatestDates
+previousLatestDates f = do
+  let latestfile = latestDatesFileFor f
+  exists <- doesFileExist latestfile
+  if exists
+  then map (parsedate . strip) . lines . strip . T.unpack <$> readFileStrictly latestfile
+  else return []
+
+-- | Where to save latest transaction dates for the given file path.
+-- (.latest.FILE)
+latestDatesFileFor :: FilePath -> FilePath
+latestDatesFileFor f = dir </> ".latest" <.> fname
+  where
+    (dir, fname) = splitFileName f
+
+readFileStrictly :: FilePath -> IO Text
+readFileStrictly f = readFile' f >>= \t -> C.evaluate (T.length t) >> return t
+
+-- | Given zero or more latest dates (all the same, representing the
+-- latest previously seen transaction date, and how many transactions
+-- were seen on that date), remove transactions with earlier dates
+-- from the journal, and the same number of transactions on the
+-- latest date, if any, leaving only transactions that we can assume
+-- are newer. Also returns the new latest dates of the new journal.
+journalFilterSinceLatestDates :: LatestDates -> Journal -> (Journal, LatestDates)
+journalFilterSinceLatestDates [] j       = (j,  latestDates $ map tdate $ jtxns j)
+journalFilterSinceLatestDates ds@(d:_) j = (j', ds')
+  where
+    samedateorlaterts     = filter ((>= d).tdate) $ jtxns j
+    (samedatets, laterts) = span ((== d).tdate) $ sortBy (comparing tdate) samedateorlaterts
+    newsamedatets         = drop (length ds) samedatets
+    j'                    = j{jtxns=newsamedatets++laterts}
+    ds'                   = latestDates $ map tdate $ samedatets++laterts
+
+readJournalWithOpts :: InputOpts -> Maybe FilePath -> Text -> IO (Either String Journal)
+readJournalWithOpts iopts mfile txt =
+  tryReadersWithOpts iopts mfile specifiedorallreaders txt
+  where
+    specifiedorallreaders = maybe stablereaders (:[]) $ findReader (mformat_ iopts) mfile
+    stablereaders = filter (not.rExperimental) readers
+
+tryReadersWithOpts :: InputOpts -> Maybe FilePath -> [Reader] -> Text -> IO (Either String Journal)
+tryReadersWithOpts iopts mpath readers txt = firstSuccessOrFirstError [] readers
+  where
+    firstSuccessOrFirstError :: [String] -> [Reader] -> IO (Either String Journal)
+    firstSuccessOrFirstError [] []        = return $ Left "no readers found"
+    firstSuccessOrFirstError errs (r:rs) = do
+      dbg1IO "trying reader" (rFormat r)
+      result <- (runExceptT . (rParser r) (mrules_file_ iopts) (not $ ignore_assertions_ iopts) path) txt
+      dbg1IO "reader result" $ either id show result
+      case result of Right j -> return $ Right j                        -- success!
+                     Left e  -> firstSuccessOrFirstError (errs++[e]) rs -- keep trying
+    firstSuccessOrFirstError (e:_) []    = return $ Left e              -- none left, return first error
+    path = fromMaybe "(string)" mpath
+
+---
+
 
 -- tests
 
