@@ -13,7 +13,8 @@ Some of these might belong in Hledger.Read.JournalReader or Hledger.Read.
 -}
 
 --- * module
-{-# LANGUAGE CPP, RecordWildCards, NamedFieldPuns, NoMonoLocalBinds, ScopedTypeVariables, FlexibleContexts, TupleSections, OverloadedStrings #-}
+{-# LANGUAGE CPP, DeriveDataTypeable, RecordWildCards, NamedFieldPuns, NoMonoLocalBinds, ScopedTypeVariables, FlexibleContexts, TupleSections, OverloadedStrings #-}
+{-# LANGUAGE LambdaCase #-}
 
 module Hledger.Read.Common
 where
@@ -23,12 +24,15 @@ import Prelude.Compat hiding (readFile)
 import Control.Monad.Compat
 import Control.Monad.Except (ExceptT(..), runExceptT, throwError) --, catchError)
 import Control.Monad.State.Strict
-import Data.Char (isNumber)
+import Data.Char
+import Data.Data
+import Data.Default
 import Data.Functor.Identity
 import Data.List.Compat
 import Data.List.NonEmpty (NonEmpty(..))
 import Data.List.Split (wordsBy)
 import Data.Maybe
+import qualified Data.Map as M
 import Data.Monoid
 import Data.Text (Text)
 import qualified Data.Text as T
@@ -36,23 +40,55 @@ import Data.Time.Calendar
 import Data.Time.LocalTime
 import Safe
 import System.Time (getClockTime)
-import Text.Megaparsec hiding (parse,State)
-import Text.Megaparsec.Text
+import Text.Megaparsec.Compat
 
 import Hledger.Data
 import Hledger.Utils
 
 -- $setup
 
+-- | Various options to use when reading journal files.
+-- Similar to CliOptions.inputflags, simplifies the journal-reading functions.
+data InputOpts = InputOpts {
+     -- files_             :: [FilePath]
+     mformat_           :: Maybe StorageFormat  -- ^ a file/storage format to try, unless overridden
+                                                --   by a filename prefix. Nothing means try all.
+    ,mrules_file_       :: Maybe FilePath       -- ^ a conversion rules file to use (when reading CSV)
+    ,aliases_           :: [String]             -- ^ account name aliases to apply
+    ,anon_              :: Bool                 -- ^ do light anonymisation/obfuscation of the data 
+    ,ignore_assertions_ :: Bool                 -- ^ don't check balance assertions
+    ,new_               :: Bool                 -- ^ read only new transactions since this file was last read
+    ,new_save_          :: Bool                 -- ^ save latest new transactions state for next time
+    ,pivot_             :: String               -- ^ use the given field's value as the account name 
+ } deriving (Show, Data) --, Typeable)
+
+instance Default InputOpts where def = definputopts
+
+definputopts :: InputOpts
+definputopts = InputOpts def def def def def def True def
+
+rawOptsToInputOpts :: RawOpts -> InputOpts
+rawOptsToInputOpts rawopts = InputOpts{
+   -- files_             = map (T.unpack . stripquotes . T.pack) $ listofstringopt "file" rawopts
+   mformat_           = Nothing
+  ,mrules_file_       = maybestringopt "rules-file" rawopts
+  ,aliases_           = map (T.unpack . stripquotes . T.pack) $ listofstringopt "alias" rawopts
+  ,anon_              = boolopt "anon" rawopts
+  ,ignore_assertions_ = boolopt "ignore-assertions" rawopts
+  ,new_               = boolopt "new" rawopts
+  ,new_save_          = True
+  ,pivot_             = stringopt "pivot" rawopts
+  }
+
 --- * parsing utils
 
 -- | Run a string parser with no state in the identity monad.
-runTextParser, rtp :: TextParser Identity a -> Text -> Either (ParseError Char Dec) a
+runTextParser, rtp :: TextParser Identity a -> Text -> Either (ParseError Char MPErr) a
 runTextParser p t =  runParser p "" t
 rtp = runTextParser
 
 -- | Run a journal parser with a null journal-parsing state.
-runJournalParser, rjp :: Monad m => TextParser m a -> Text -> m (Either (ParseError Char Dec) a)
+runJournalParser, rjp :: Monad m => TextParser m a -> Text -> m (Either (ParseError Char MPErr) a)
 runJournalParser p t = runParserT p "" t
 rjp = runJournalParser
 
@@ -89,7 +125,7 @@ parseAndFinaliseJournal parser assrt f txt = do
                         Left e  -> throwError e
     Left e   -> throwError $ parseErrorPretty e
 
-parseAndFinaliseJournal' :: JournalParser ParsedJournal -> Bool -> FilePath -> Text -> ExceptT String IO Journal
+parseAndFinaliseJournal' :: JournalParser Identity ParsedJournal -> Bool -> FilePath -> Text -> ExceptT String IO Journal
 parseAndFinaliseJournal' parser assrt f txt = do
   t <- liftIO getClockTime
   y <- liftIO getCurrentYear
@@ -100,32 +136,50 @@ parseAndFinaliseJournal' parser assrt f txt = do
                         Left e  -> throwError e
     Left e   -> throwError $ parseErrorPretty e
 
-setYear :: Year -> JournalStateParser m ()
+setYear :: Year -> JournalParser m ()
 setYear y = modify' (\j -> j{jparsedefaultyear=Just y})
 
-getYear :: JournalStateParser m (Maybe Year)
+getYear :: JournalParser m (Maybe Year)
 getYear = fmap jparsedefaultyear get
 
-setDefaultCommodityAndStyle :: (CommoditySymbol,AmountStyle) -> JournalStateParser m ()
+setDefaultCommodityAndStyle :: (CommoditySymbol,AmountStyle) -> JournalParser m ()
 setDefaultCommodityAndStyle cs = modify' (\j -> j{jparsedefaultcommodity=Just cs})
 
-getDefaultCommodityAndStyle :: JournalStateParser m (Maybe (CommoditySymbol,AmountStyle))
+getDefaultCommodityAndStyle :: JournalParser m (Maybe (CommoditySymbol,AmountStyle))
 getDefaultCommodityAndStyle = jparsedefaultcommodity `fmap` get
 
-pushAccount :: AccountName -> JournalStateParser m ()
+-- | Get amount style associated with default currency.
+--
+-- Returns 'AmountStyle' used to defined by a latest default commodity directive
+-- prior to current position within this file or its parents.
+getDefaultAmountStyle :: JournalParser m (Maybe AmountStyle)
+getDefaultAmountStyle = fmap snd <$> getDefaultCommodityAndStyle
+
+-- | Lookup currency-specific amount style.
+--
+-- Returns 'AmountStyle' used in commodity directive within current journal
+-- prior to current position or in its parents files.
+getAmountStyle :: CommoditySymbol -> JournalParser m (Maybe AmountStyle)
+getAmountStyle commodity = do
+    specificStyle <-  maybe Nothing cformat . M.lookup commodity . jcommodities <$> get
+    defaultStyle <- fmap snd <$> getDefaultCommodityAndStyle
+    let effectiveStyle = listToMaybe $ catMaybes [specificStyle, defaultStyle]
+    return effectiveStyle
+
+pushAccount :: AccountName -> JournalParser m ()
 pushAccount acct = modify' (\j -> j{jaccounts = acct : jaccounts j})
 
-pushParentAccount :: AccountName -> JournalStateParser m ()
+pushParentAccount :: AccountName -> JournalParser m ()
 pushParentAccount acct = modify' (\j -> j{jparseparentaccounts = acct : jparseparentaccounts j})
 
-popParentAccount :: JournalStateParser m ()
+popParentAccount :: JournalParser m ()
 popParentAccount = do
   j <- get
   case jparseparentaccounts j of
     []       -> unexpected (Tokens ('E' :| "nd of apply account block with no beginning"))
     (_:rest) -> put j{jparseparentaccounts=rest}
 
-getParentAccount :: JournalStateParser m AccountName
+getParentAccount :: JournalParser m AccountName
 getParentAccount = fmap (concatAccountNames . reverse . jparseparentaccounts) get
 
 addAccountAlias :: MonadState Journal m => AccountAlias -> m ()
@@ -169,19 +223,19 @@ parserErrorAt pos s = throwError $ sourcePosPretty pos ++ ":\n" ++ s
 --- * parsers
 --- ** transaction bits
 
-statusp :: TextParser m ClearedStatus
+statusp :: TextParser m Status
 statusp =
   choice'
     [ many spacenonewline >> char '*' >> return Cleared
     , many spacenonewline >> char '!' >> return Pending
-    , return Uncleared
+    , return Unmarked
     ]
     <?> "cleared status"
 
 codep :: TextParser m String
 codep = try (do { some spacenonewline; char '(' <?> "codep"; anyChar `manyTill` char ')' } ) <|> return ""
 
-descriptionp :: JournalStateParser m String
+descriptionp :: JournalParser m String
 descriptionp = many (noneOf (";\n" :: [Char]))
 
 --- ** dates
@@ -190,7 +244,7 @@ descriptionp = many (noneOf (";\n" :: [Char]))
 -- Hyphen (-) and period (.) are also allowed as separators.
 -- The year may be omitted if a default year has been set.
 -- Leading zeroes may be omitted.
-datep :: JournalStateParser m Day
+datep :: JournalParser m Day
 datep = do
   -- hacky: try to ensure precise errors for invalid dates
   -- XXX reported error position is not too good
@@ -220,7 +274,7 @@ datep = do
 -- Seconds are optional.
 -- The timezone is optional and ignored (the time is always interpreted as a local time).
 -- Leading zeroes may be omitted (except in a timezone).
-datetimep :: JournalStateParser m LocalTime
+datetimep :: JournalParser m LocalTime
 datetimep = do
   day <- datep
   lift $ some spacenonewline
@@ -248,7 +302,7 @@ datetimep = do
   -- return $ localTimeToUTC tz' $ LocalTime day $ TimeOfDay h' m' (fromIntegral s')
   return $ LocalTime day $ TimeOfDay h' m' (fromIntegral s')
 
-secondarydatep :: Day -> JournalStateParser m Day
+secondarydatep :: Day -> JournalParser m Day
 secondarydatep primarydate = do
   char '='
   -- kludgy way to use primary date for default year
@@ -274,7 +328,7 @@ secondarydatep primarydate = do
 --- ** account names
 
 -- | Parse an account name, then apply any parent account prefix and/or account aliases currently in effect.
-modifiedaccountnamep :: JournalStateParser m AccountName
+modifiedaccountnamep :: JournalParser m AccountName
 modifiedaccountnamep = do
   parent <- getParentAccount
   aliases <- getAccountAliases
@@ -313,7 +367,7 @@ accountnamep = do
 -- | Parse whitespace then an amount, with an optional left or right
 -- currency symbol and optional price, or return the special
 -- "missing" marker amount.
-spaceandamountormissingp :: Monad m => JournalStateParser m MixedAmount
+spaceandamountormissingp :: Monad m => JournalParser m MixedAmount
 spaceandamountormissingp =
   try (do
         lift $ some spacenonewline
@@ -337,7 +391,7 @@ test_spaceandamountormissingp = do
 -- | Parse a single-commodity amount, with optional symbol on the left or
 -- right, optional unit or total price, and optional (ignored)
 -- ledger-style balance assertion or fixed lot price declaration.
-amountp :: Monad m => JournalStateParser m Amount
+amountp :: Monad m => JournalParser m Amount
 amountp = try leftsymbolamountp <|> try rightsymbolamountp <|> nosymbolamountp
 
 #ifdef TESTS
@@ -371,38 +425,52 @@ signp = do
   return $ case sign of Just '-' -> "-"
                         _        -> ""
 
-leftsymbolamountp :: Monad m => JournalStateParser m Amount
+multiplierp :: TextParser m Bool
+multiplierp = do
+  multiplier <- optional $ oneOf ("*" :: [Char])
+  return $ case multiplier of Just '*' -> True
+                              _        -> False
+
+leftsymbolamountp :: Monad m => JournalParser m Amount
 leftsymbolamountp = do
   sign <- lift signp
+  m <- lift multiplierp
   c <- lift commoditysymbolp
+  suggestedStyle <- getAmountStyle c
   sp <- lift $ many spacenonewline
-  (q,prec,mdec,mgrps) <- lift numberp
+  (q,prec,mdec,mgrps) <- lift $ numberp suggestedStyle
   let s = amountstyle{ascommodityside=L, ascommodityspaced=not $ null sp, asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
   p <- priceamountp
   let applysign = if sign=="-" then negate else id
-  return $ applysign $ Amount c q p s
+  return $ applysign $ Amount c q p s m
   <?> "left-symbol amount"
 
-rightsymbolamountp :: Monad m => JournalStateParser m Amount
+rightsymbolamountp :: Monad m => JournalParser m Amount
 rightsymbolamountp = do
-  (q,prec,mdec,mgrps) <- lift numberp
+  m <- lift multiplierp
+  sign <- lift signp
+  rawnum <- lift $ rawnumberp
   sp <- lift $ many spacenonewline
   c <- lift commoditysymbolp
+  suggestedStyle <- getAmountStyle c
+  let (q,prec,mdec,mgrps) = fromRawNumber suggestedStyle (sign == "-") rawnum
   p <- priceamountp
   let s = amountstyle{ascommodityside=R, ascommodityspaced=not $ null sp, asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps}
-  return $ Amount c q p s
+  return $ Amount c q p s m
   <?> "right-symbol amount"
 
-nosymbolamountp :: Monad m => JournalStateParser m Amount
+nosymbolamountp :: Monad m => JournalParser m Amount
 nosymbolamountp = do
-  (q,prec,mdec,mgrps) <- lift numberp
+  m <- lift multiplierp
+  suggestedStyle <- getDefaultAmountStyle
+  (q,prec,mdec,mgrps) <- lift $ numberp suggestedStyle
   p <- priceamountp
   -- apply the most recently seen default commodity and style to this commodityless amount
   defcs <- getDefaultCommodityAndStyle
   let (c,s) = case defcs of
         Just (defc,defs) -> (defc, defs{asprecision=max (asprecision defs) prec})
         Nothing          -> ("", amountstyle{asprecision=prec, asdecimalpoint=mdec, asdigitgroups=mgrps})
-  return $ Amount c q p s
+  return $ Amount c q p s m
   <?> "no-symbol amount"
 
 commoditysymbolp :: TextParser m CommoditySymbol
@@ -418,7 +486,7 @@ quotedcommoditysymbolp = do
 simplecommoditysymbolp :: TextParser m CommoditySymbol
 simplecommoditysymbolp = T.pack <$> some (noneOf nonsimplecommoditychars)
 
-priceamountp :: Monad m => JournalStateParser m Price
+priceamountp :: Monad m => JournalParser m Price
 priceamountp =
     try (do
           lift (many spacenonewline)
@@ -434,14 +502,15 @@ priceamountp =
             return $ UnitPrice a))
          <|> return NoPrice
 
-partialbalanceassertionp :: Monad m => JournalStateParser m (Maybe Amount)
+partialbalanceassertionp :: Monad m => JournalParser m BalanceAssertion
 partialbalanceassertionp =
     try (do
           lift (many spacenonewline)
+          sourcepos <- genericSourcePos <$> lift getPosition
           char '='
           lift (many spacenonewline)
           a <- amountp -- XXX should restrict to a simple amount
-          return $ Just $ a)
+          return $ Just (a, sourcepos))
          <|> return Nothing
 
 -- balanceassertion :: Monad m => TextParser m (Maybe MixedAmount)
@@ -455,7 +524,7 @@ partialbalanceassertionp =
 --          <|> return Nothing
 
 -- http://ledger-cli.org/3.0/doc/ledger3.html#Fixing-Lot-Prices
-fixedlotpricep :: Monad m => JournalStateParser m (Maybe Amount)
+fixedlotpricep :: Monad m => JournalParser m (Maybe Amount)
 fixedlotpricep =
     try (do
           lift (many spacenonewline)
@@ -481,55 +550,79 @@ fixedlotpricep =
 -- seen following the decimal point), the decimal point character used if any,
 -- and the digit group style if any.
 --
-numberp :: TextParser m (Quantity, Int, Maybe Char, Maybe DigitGroupStyle)
-numberp = do
-  -- a number is an optional sign followed by a sequence of digits possibly
-  -- interspersed with periods, commas, or both
-  -- ptrace "numberp"
-  sign <- signp
-  parts <- some $ choice' [some digitChar, some $ char ',', some $ char '.']
-  dbg8 "numberp parsed" (sign,parts) `seq` return ()
+numberp :: Maybe AmountStyle -> TextParser m (Quantity, Int, Maybe Char, Maybe DigitGroupStyle)
+numberp suggestedStyle = do
+    -- a number is an optional sign followed by a sequence of digits possibly
+    -- interspersed with periods, commas, or both
+    -- ptrace "numberp"
+    sign <- signp
+    raw <- rawnumberp
+    dbg8 "numberp parsed" raw `seq` return ()
+    return $ dbg8 "numberp quantity,precision,mdecimalpoint,mgrps" (fromRawNumber suggestedStyle (sign == "-") raw)
+    <?> "numberp"
 
-  -- check the number is well-formed and identify the decimal point and digit
-  -- group separator characters used, if any
-  let (numparts, puncparts) = partition numeric parts
-      (ok, mdecimalpoint, mseparator) =
-          case (numparts, puncparts) of
-            ([],_)     -> (False, Nothing, Nothing)  -- no digits, not ok
-            (_,[])     -> (True, Nothing, Nothing)   -- digits with no punctuation, ok
-            (_,[[d]])  -> (True, Just d, Nothing)    -- just a single punctuation of length 1, assume it's a decimal point
-            (_,[_])    -> (False, Nothing, Nothing)  -- a single punctuation of some other length, not ok
-            (_,_:_:_)  ->                                       -- two or more punctuations
-              let (s:ss, d) = (init puncparts, last puncparts)  -- the leftmost is a separator and the rightmost may be a decimal point
-              in if any ((/=1).length) puncparts               -- adjacent punctuation chars, not ok
-                    || any (s/=) ss                            -- separator chars vary, not ok
-                    || head parts == s                        -- number begins with a separator char, not ok
-                 then (False, Nothing, Nothing)
-                 else if s == d
-                      then (True, Nothing, Just $ head s)       -- just one kind of punctuation - must be separators
-                      else (True, Just $ head d, Just $ head s) -- separator(s) and a decimal point
-  unless ok $ fail $ "number seems ill-formed: "++concat parts
+fromRawNumber :: Maybe AmountStyle -> Bool -> (Maybe Char, [String], Maybe (Char, String)) -> (Quantity, Int, Maybe Char, Maybe DigitGroupStyle)
+fromRawNumber suggestedStyle negated raw = (quantity, precision, mdecimalpoint, mgrps) where
+    -- unpack with a hint if useful
+    (mseparator, intparts, mdecimalpoint, frac) =
+            case raw of
+                -- just a single punctuation between two digits groups, assume it's a decimal point
+                (Just s, [firstGroup, lastGroup], Nothing)
+                    -- if have a decimalHint restrict this assumpion only to a matching separator
+                    | maybe True (`asdecimalcheck` s) suggestedStyle -> (Nothing, [firstGroup], Just s, lastGroup)
 
-  -- get the digit group sizes and digit group style if any
-  let (intparts',fracparts') = span ((/= mdecimalpoint) . Just . head) parts
-      (intparts, fracpart) = (filter numeric intparts', filter numeric fracparts')
-      groupsizes = reverse $ case map length intparts of
+                (firstSep, digitGroups, Nothing) -> (firstSep, digitGroups, Nothing, [])
+                (firstSep, digitGroups, Just (d, frac)) -> (firstSep, digitGroups, Just d, frac)
+
+    -- get the digit group sizes and digit group style if any
+    groupsizes = reverse $ case map length intparts of
                                (a:b:cs) | a < b -> b:cs
                                gs               -> gs
-      mgrps = (`DigitGroups` groupsizes) <$> mseparator
+    mgrps = (`DigitGroups` groupsizes) <$> mseparator
 
-  -- put the parts back together without digit group separators, get the precision and parse the value
-  let int = concat $ "":intparts
-      frac = concat $ "":fracpart
-      precision = length frac
-      int' = if null int then "0" else int
-      frac' = if null frac then "0" else frac
-      quantity = read $ sign++int'++"."++frac' -- this read should never fail
+    -- put the parts back together without digit group separators, get the precision and parse the value
+    repr = (if negated then "-" else "") ++ "0" ++ concat intparts ++ (if null frac then "" else "." ++ frac)
+    quantity = read repr
+    precision = length frac
 
-  return $ dbg8 "numberp quantity,precision,mdecimalpoint,mgrps" (quantity,precision,mdecimalpoint,mgrps)
-  <?> "numberp"
-  where
-    numeric = isNumber . headDef '_'
+    asdecimalcheck :: AmountStyle -> Char -> Bool
+    asdecimalcheck = \case
+        AmountStyle{asdecimalpoint = Just d} -> (d ==)
+        AmountStyle{asdigitgroups = Just (DigitGroups g _)} -> (g /=)
+        AmountStyle{asprecision = 0} -> const False
+        _ -> const True
+
+
+rawnumberp :: TextParser m (Maybe Char, [String], Maybe (Char, String))
+rawnumberp = do
+    let sepChars = ['.', ','] -- all allowed punctuation characters
+
+    (firstSep, groups) <- option (Nothing, []) $ do
+        leadingDigits <- some digitChar
+        option (Nothing, [leadingDigits]) . try $ do
+            firstSep <- oneOf sepChars <|> whitespaceChar
+            groups <- some digitChar `sepBy1` char firstSep
+            return (Just firstSep, leadingDigits : groups)
+
+    let remSepChars = maybe sepChars (`delete` sepChars) firstSep
+        modifier
+            | null groups = fmap Just  -- if no digits so far, we require at least some decimals
+            | otherwise = optional
+
+    extraGroup <- modifier $ do
+        lastSep <- oneOf remSepChars
+        digits <- modifier $ some digitChar  -- decimal separator allowed to be without digits if had some before
+        return (lastSep, fromMaybe [] digits)
+
+    -- make sure we didn't leading part of mistyped number
+    notFollowedBy $ oneOf sepChars <|> (whitespaceChar >> digitChar)
+
+    return $ dbg8 "rawnumberp" (firstSep, groups, extraGroup)
+    <?> "rawnumberp"
+
+-- | Parse a unicode char that represents any non-control space char (Zs general category).
+whitespaceChar :: TextParser m Char
+whitespaceChar = charCategory Space
 
 -- test_numberp = do
 --       let s `is` n = assertParseEqual (parseWithState mempty numberp s) n
@@ -555,7 +648,7 @@ numberp = do
 
 --- ** comments
 
-multilinecommentp :: JournalStateParser m ()
+multilinecommentp :: JournalParser m ()
 multilinecommentp = do
   string "comment" >> lift (many spacenonewline) >> newline
   go
@@ -564,17 +657,17 @@ multilinecommentp = do
          <|> (anyLine >> go)
     anyLine = anyChar `manyTill` newline
 
-emptyorcommentlinep :: JournalStateParser m ()
+emptyorcommentlinep :: JournalParser m ()
 emptyorcommentlinep = do
-  lift (many spacenonewline) >> (commentp <|> (lift (many spacenonewline) >> newline >> return ""))
+  lift (many spacenonewline) >> (linecommentp <|> (lift (many spacenonewline) >> newline >> return ""))
   return ()
 
 -- | Parse a possibly multi-line comment following a semicolon.
-followingcommentp :: JournalStateParser m Text
+followingcommentp :: JournalParser m Text
 followingcommentp =
   -- ptrace "followingcommentp"
-  do samelinecomment <- lift (many spacenonewline) >> (try semicoloncommentp <|> (newline >> return ""))
-     newlinecomments <- many (try (lift (some spacenonewline) >> semicoloncommentp))
+  do samelinecomment <- lift (many spacenonewline) >> (try commentp <|> (newline >> return ""))
+     newlinecomments <- many (try (lift (some spacenonewline) >> commentp))
      return $ T.unlines $ samelinecomment:newlinecomments
 
 -- | Parse a possibly multi-line comment following a semicolon, and
@@ -606,10 +699,10 @@ followingcommentandtagsp mdefdate = do
   -- to get good error positions.
   startpos <- getPosition
   commentandwhitespace :: String <- do
-    let semicoloncommentp' = (:) <$> char ';' <*> anyChar `manyTill` eolof
+    let commentp' = (:) <$> char ';' <*> anyChar `manyTill` eolof
     sp1 <- lift (many spacenonewline)
-    l1  <- try (lift semicoloncommentp') <|> (newline >> return "")
-    ls  <- lift . many $ try ((++) <$> some spacenonewline <*> semicoloncommentp')
+    l1  <- try (lift commentp') <|> (newline >> return "")
+    ls  <- lift . many $ try ((++) <$> some spacenonewline <*> commentp')
     return $ unlines $ (sp1 ++ l1) : ls
   let comment = T.pack $ unlines $ map (lstrip . dropWhile (==';') . strip) $ lines commentandwhitespace
   -- pdbg 0 $ "commentws:"++show commentandwhitespace
@@ -632,16 +725,17 @@ followingcommentandtagsp mdefdate = do
 
   return (comment, tags, mdate, mdate2)
 
-commentp :: JournalStateParser m Text
-commentp = commentStartingWithp commentchars
+-- A transaction/posting comment must start with a semicolon.
+-- This parser ignores leading whitespace.
+commentp :: JournalParser m Text
+commentp = commentStartingWithp ";"
 
-commentchars :: [Char]
-commentchars = "#;*"
+-- A line (file-level) comment can start with a semicolon, hash,
+-- or star (allowing org nodes). This parser ignores leading whitespace.
+linecommentp :: JournalParser m Text
+linecommentp = commentStartingWithp ";#*" 
 
-semicoloncommentp :: JournalStateParser m Text
-semicoloncommentp = commentStartingWithp ";"
-
-commentStartingWithp :: [Char] -> JournalStateParser m Text
+commentStartingWithp :: [Char] -> JournalParser m Text
 commentStartingWithp cs = do
   -- ptrace "commentStartingWith"
   oneOf cs
@@ -672,7 +766,7 @@ commentTags s =
     Left _  -> [] -- shouldn't happen
 
 -- | Parse all tags found in a string.
-tagsp :: Parser [Tag]
+tagsp :: SimpleTextParser [Tag]
 tagsp = -- do
   -- pdbg 0 $ "tagsp"
   many (try (nontagp >> tagp))
@@ -681,7 +775,7 @@ tagsp = -- do
 --
 -- >>> rtp nontagp "\na b:, \nd:e, f"
 -- Right "\na "
-nontagp :: Parser String
+nontagp :: SimpleTextParser String
 nontagp = -- do
   -- pdbg 0 "nontagp"
   -- anyChar `manyTill` (lookAhead (try (tagorbracketeddatetagsp Nothing >> return ()) <|> eof))
@@ -695,7 +789,7 @@ nontagp = -- do
 -- >>> rtp tagp "a:b b , c AuxDate: 4/2"
 -- Right ("a","b b")
 --
-tagp :: Parser Tag
+tagp :: SimpleTextParser Tag
 tagp = do
   -- pdbg 0 "tagp"
   n <- tagnamep
@@ -705,7 +799,7 @@ tagp = do
 -- |
 -- >>> rtp tagnamep "a:"
 -- Right "a"
-tagnamep :: Parser Text
+tagnamep :: SimpleTextParser Text
 tagnamep = -- do
   -- pdbg 0 "tagnamep"
   T.pack <$> some (noneOf (": \t\n" :: [Char])) <* char ':'
@@ -752,13 +846,13 @@ datetagp :: Monad m => Maybe Day -> ErroringJournalParser m (TagName,Day)
 datetagp mdefdate = do
   -- pdbg 0 "datetagp"
   string "date"
-  n <- T.pack . fromMaybe "" <$> optional (string "2")
+  n <- fromMaybe "" <$> optional (mptext "2")
   char ':'
   startpos <- getPosition
   v <- lift tagvaluep
   -- re-parse value as a date.
   j <- get
-  let ep :: Either (ParseError Char Dec) Day
+  let ep :: Either (ParseError Char MPErr) Day
       ep = parseWithState'
              j{jparsedefaultyear=first3.toGregorian <$> mdefdate}
              -- The value extends to a comma, newline, or end of file.
@@ -818,7 +912,7 @@ bracketeddatetagsp mdefdate = do
   -- looks sufficiently like a bracketed date, now we
   -- re-parse as dates and throw any errors
   j <- get
-  let ep :: Either (ParseError Char Dec) (Maybe Day, Maybe Day)
+  let ep :: Either (ParseError Char MPErr) (Maybe Day, Maybe Day)
       ep = parseWithState'
              j{jparsedefaultyear=first3.toGregorian <$> mdefdate}
              (do
