@@ -1,9 +1,9 @@
-{-# LANGUAGE CPP #-}
 {-|
 
 A reader for CSV data, using an extra rules file to help interpret the data.
 
 -}
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeFamilies #-}
@@ -38,9 +38,9 @@ import Data.Char (toLower, isDigit, isSpace)
 import Data.List.Compat
 import Data.Maybe
 import Data.Ord
-import qualified Data.Set as S
 import Data.Text (Text)
 import qualified Data.Text as T
+import qualified Data.Text.IO as T
 import Data.Time.Calendar (Day)
 #if MIN_VERSION_time(1,5,0)
 import Data.Time.Format (parseTimeM, defaultTimeLocale)
@@ -51,13 +51,11 @@ import System.Locale (defaultTimeLocale)
 import Safe
 import System.Directory (doesFileExist)
 import System.FilePath
-import System.IO (stderr)
 import Test.HUnit hiding (State)
 import Text.CSV (parseCSV, CSV)
-import Text.Megaparsec hiding (parse, State)
-import Text.Megaparsec.Text
+import Text.Megaparsec.Compat hiding (parse)
 import qualified Text.Parsec as Parsec
-import Text.Printf (hPrintf,printf)
+import Text.Printf (printf)
 
 import Hledger.Data
 import Hledger.Utils.UTF8IOCompat (getContents)
@@ -105,7 +103,7 @@ readJournalFromCsv mrulesfile csvfile csvdata =
   rulestext <-
     if rulesfileexists
     then do
-      hPrintf stderr "using conversion rules file %s\n" rulesfile
+      dbg1IO "using conversion rules file" rulesfile
       liftIO $ (readFile' rulesfile >>= expandIncludes (takeDirectory rulesfile))
     else return $ defaultRulesText rulesfile
   rules <- liftIO (runExceptT $ parseAndValidateCsvRules rulesfile rulestext) >>= either throwerr return 
@@ -130,27 +128,40 @@ readJournalFromCsv mrulesfile csvfile csvdata =
   -- let (headerlines, datalines) = identifyHeaderLines records
   --     mfieldnames = lastMay headerlines
 
-  -- convert to transactions and return as a journal
-  let txns = snd $ mapAccumL
-                     (\pos r -> (pos,
-                                 transactionFromCsvRecord
-                                   (let SourcePos name line col =  pos in
-                                    SourcePos name (unsafePos $ unPos line + 1) col)
-                                   rules
-                                    r))
-                     (initialPos parsecfilename) records
+  let 
+    -- convert CSV records to transactions
+    txns = snd $ mapAccumL
+                   (\pos r -> 
+                      let
+                        SourcePos name line col = pos
+                        line' = (mpMkPos . (+1) . mpUnPos) line
+                        pos' = SourcePos name line' col
+                      in
+                        (pos, transactionFromCsvRecord pos' rules r)
+                   )
+                   (initialPos parsecfilename) records
 
-  -- heuristic: if the records appear to have been in reverse date order,
-  -- reverse them all as well as doing a txn date sort,
-  -- so that same-day txns' original order is preserved
-      txns' | length txns > 1 && tdate (head txns) > tdate (last txns) = reverse txns
-            | otherwise = txns
+    -- Ensure transactions are ordered chronologically.
+    -- First, reverse them to get same-date transactions ordered chronologically,
+    -- if the CSV records seem to be most-recent-first, ie if there's an explicit 
+    -- "newest-first" directive, or if there's more than one date and the first date
+    -- is more recent than the last.
+    txns' = 
+      (if newestfirst || mseemsnewestfirst == Just True then reverse else id) txns
+      where
+        newestfirst = dbg3 "newestfirst" $ isJust $ getDirective "newest-first" rules
+        mseemsnewestfirst = dbg3 "mseemsnewestfirst" $  
+          case nub $ map tdate txns of 
+            ds | length ds > 1 -> Just $ head ds > last ds 
+            _                  -> Nothing
+    -- Second, sort by date.
+    txns'' = sortBy (comparing tdate) txns'
 
   when (not rulesfileexists) $ do
-    hPrintf stderr "created default conversion rules file %s, edit this for better results\n" rulesfile
+    dbg1IO "creating conversion rules file" rulesfile
     writeFile rulesfile $ T.unpack rulestext
 
-  return $ Right nulljournal{jtxns=sortBy (comparing tdate) txns'}
+  return $ Right nulljournal{jtxns=txns''}
 
 parseCsv :: FilePath -> String -> IO (Either Parsec.ParseError CSV)
 parseCsv path csvdata =
@@ -209,6 +220,7 @@ defaultRulesText csvfile = T.pack $ unlines
   ,"fields date, description, amount"
   ,""
   ,"#skip 1"
+  ,"#newest-first"
   ,""
   ,"#date-format %-d/%-m/%Y"
   ,"#date-format %-m/%-d/%Y"
@@ -231,7 +243,7 @@ Grammar for the CSV conversion rules, more or less:
 
 RULES: RULE*
 
-RULE: ( FIELD-LIST | FIELD-ASSIGNMENT | CONDITIONAL-BLOCK | SKIP | DATE-FORMAT | COMMENT | BLANK ) NEWLINE
+RULE: ( FIELD-LIST | FIELD-ASSIGNMENT | CONDITIONAL-BLOCK | SKIP | NEWEST-FIRST | DATE-FORMAT | COMMENT | BLANK ) NEWLINE
 
 FIELD-LIST: fields SPACE FIELD-NAME ( SPACE? , SPACE? FIELD-NAME )*
 
@@ -300,7 +312,7 @@ data CsvRules = CsvRules {
   rconditionalblocks :: [ConditionalBlock]
 } deriving (Show, Eq)
 
-type CsvRulesParser a = StateT CsvRules Parser a
+type CsvRulesParser a = StateT CsvRules SimpleTextParser a
 
 type DirectiveName    = String
 type CsvFieldName     = String
@@ -355,22 +367,18 @@ parseRulesFile :: FilePath -> ExceptT String IO CsvRules
 parseRulesFile f = 
   liftIO (readFile' f >>= expandIncludes (takeDirectory f)) >>= parseAndValidateCsvRules f
 
--- | Look for hledger rules file-style include directives in this text,
--- and interpolate the included files, recursively.
--- Included file paths may be relative to the directory of the
--- provided file path.
+-- | Inline all files referenced by include directives in this hledger CSV rules text, recursively.
+-- Included file paths may be relative to the directory of the provided file path.
 -- This is a cheap hack to avoid rewriting the CSV rules parser.
-expandIncludes :: FilePath -> T.Text -> IO T.Text
-expandIncludes basedir content = do
-  let (ls,rest) = break (T.isPrefixOf "include") $ T.lines content
-  case rest of
-    [] -> return $ T.unlines ls
-    ((T.stripPrefix "include" -> Just f):ls') -> do
-      let f'       = basedir </> dropWhile isSpace (T.unpack f)
-          basedir' = takeDirectory f'
-      included <- readFile' f' >>= expandIncludes basedir'
-      return $ T.unlines [T.unlines ls, included, T.unlines ls']
-    ls' -> return $ T.unlines $ ls ++ ls'   -- should never get here
+expandIncludes dir content = mapM (expandLine dir) (T.lines content) >>= return . T.unlines
+  where
+    expandLine dir line =
+      case line of
+        (T.stripPrefix "include " -> Just f) -> expandIncludes dir' =<< T.readFile f'
+          where
+            f' = dir </> dropWhile isSpace (T.unpack f)
+            dir' = takeDirectory f'
+        _ -> return line 
 
 -- | An error-throwing action that parses this text as CSV conversion rules 
 -- and runs some extra validation checks. The file path is for error messages.
@@ -382,14 +390,11 @@ parseAndValidateCsvRules rulesfile s = do
     Right r -> do
                r_ <- liftIO $ runExceptT $ validateRules r
                ExceptT $ case r_ of
-                 Left e -> return $ Left $ parseErrorPretty $ toParseError e
+                 Left  s -> return $ Left $ parseErrorPretty $ mpMkParseError rulesfile s
                  Right r -> return $ Right r
-  where
-    toParseError :: forall s. Ord s => s -> ParseError Char s
-    toParseError s = (mempty :: ParseError Char s) { errorCustom = S.singleton s}
 
 -- | Parse this text as CSV conversion rules. The file path is for error messages.
-parseCsvRules :: FilePath -> T.Text -> Either (ParseError Char Dec) CsvRules
+parseCsvRules :: FilePath -> T.Text -> Either (ParseError Char MPErr) CsvRules
 -- parseCsvRules rulesfile s = runParser csvrulesfile nullrules{baseAccount=takeBaseName rulesfile} rulesfile s
 parseCsvRules rulesfile s =
   runParser (evalStateT rulesp rules) rulesfile s
@@ -441,10 +446,10 @@ commentcharp = oneOf (";#*" :: [Char])
 directivep :: CsvRulesParser (DirectiveName, String)
 directivep = (do
   lift $ pdbg 3 "trying directive"
-  d <- choiceInState $ map string directives
+  d <- fmap T.unpack $ choiceInState $ map (lift . mptext . T.pack) directives
   v <- (((char ':' >> lift (many spacenonewline)) <|> lift (some spacenonewline)) >> directivevalp)
        <|> (optional (char ':') >> lift (many spacenonewline) >> lift eolof >> return "")
-  return (d,v)
+  return (d, v)
   ) <?> "directive"
 
 directives =
@@ -453,6 +458,7 @@ directives =
   -- ,"default-currency"
   -- ,"skip-lines" -- old
   ,"skip"
+  ,"newest-first"
    -- ,"base-account"
    -- ,"base-currency"
   ]
@@ -496,23 +502,27 @@ fieldassignmentp = do
   <?> "field assignment"
 
 journalfieldnamep :: CsvRulesParser String
-journalfieldnamep = lift (pdbg 2 "trying journalfieldnamep") >> choiceInState (map string journalfieldnames)
+journalfieldnamep = do
+  lift (pdbg 2 "trying journalfieldnamep")
+  T.unpack <$> choiceInState (map (lift . mptext . T.pack) journalfieldnames)
 
-journalfieldnames =
-  [-- pseudo fields:
-   "amount-in"
+-- Transaction fields and pseudo fields for CSV conversion. 
+-- Names must precede any other name they contain, for the parser 
+-- (amount-in before amount; date2 before date). TODO: fix
+journalfieldnames = [
+   "account1"
+  ,"account2"
+  ,"amount-in"
   ,"amount-out"
+  ,"amount"
+  ,"balance"
+  ,"code"
+  ,"comment"
   ,"currency"
-   -- standard fields:
   ,"date2"
   ,"date"
-  ,"status"
-  ,"code"
   ,"description"
-  ,"amount"
-  ,"account1"
-  ,"account2"
-  ,"comment"
+  ,"status"
   ]
 
 assignmentseparatorp :: CsvRulesParser ()
@@ -554,7 +564,7 @@ recordmatcherp = do
   <?> "record matcher"
 
 matchoperatorp :: CsvRulesParser String
-matchoperatorp = choiceInState $ map string
+matchoperatorp = fmap T.unpack $ choiceInState $ map mptext
   ["~"
   -- ,"!~"
   -- ,"="
@@ -625,7 +635,7 @@ transactionFromCsvRecord sourcepos rules record = t
       ]
     status      =
       case mfieldtemplate "status" of
-        Nothing  -> Uncleared
+        Nothing  -> Unmarked
         Just str -> either statuserror id .
                     runParser (statusp <* eof) "" .
                     T.pack $ render str
@@ -639,10 +649,10 @@ transactionFromCsvRecord sourcepos rules record = t
     comment     = maybe "" render $ mfieldtemplate "comment"
     precomment  = maybe "" render $ mfieldtemplate "precomment"
     currency    = maybe (fromMaybe "" mdefaultcurrency) render $ mfieldtemplate "currency"
-    amountstr   = (currency++) $ negateIfParenthesised $ getAmountStr rules record
-    amount      = either amounterror (Mixed . (:[])) $ runParser (evalStateT (amountp <* eof) mempty) "" $ T.pack amountstr
+    amountstr   = (currency++) <$> simplifySign <$> getAmountStr rules record
+    maybeamount      = either amounterror (Mixed . (:[])) <$> runParser (evalStateT (amountp <* eof) mempty) "" <$> T.pack <$> amountstr
     amounterror err = error' $ unlines
-      ["error: could not parse \""++amountstr++"\" as an amount"
+      ["error: could not parse \""++fromJust amountstr++"\" as an amount"
       ,showRecord record
       ,"the amount rule is:      "++(fromMaybe "" $ mfieldtemplate "amount")
       ,"the currency rule is:    "++(fromMaybe "unspecified" $ mfieldtemplate "currency")
@@ -652,10 +662,13 @@ transactionFromCsvRecord sourcepos rules record = t
        ++"change your amount or currency rules, "
        ++"or "++maybe "add a" (const "change your") mskip++" skip rule"
       ]
-    -- Using costOfMixedAmount here to allow complex costs like "10 GBP @@ 15 USD".
-    -- Aim is to have "10 GBP @@ 15 USD" applied to account2, but have "-15USD" applied to account1
-    amount1        = costOfMixedAmount amount
-    amount2        = (-amount)
+    amount1 = case maybeamount of
+                Just a -> a
+                Nothing | balance /= Nothing -> nullmixedamt
+                Nothing -> error' $ "amount and balance have no value\n"++showRecord record
+    -- convert balancing amount to cost like hledger print, so eg if
+    -- amount1 is "10 GBP @@ 15 USD", amount2 will be "-15 USD".
+    amount2        = costOfMixedAmount (-amount1)
     s `or` def  = if null s then def else s
     defaccount1 = fromMaybe "unknown" $ mdirective "default-account1"
     defaccount2 = case isNegativeMixedAmount amount2 of
@@ -663,6 +676,18 @@ transactionFromCsvRecord sourcepos rules record = t
                    _         -> "expenses:unknown"
     account1    = T.pack $ maybe "" render (mfieldtemplate "account1") `or` defaccount1
     account2    = T.pack $ maybe "" render (mfieldtemplate "account2") `or` defaccount2
+    balance     = maybe Nothing (parsebalance.render) $ mfieldtemplate "balance"
+    parsebalance str 
+      | all isSpace str  = Nothing
+      | otherwise = Just $ (either (balanceerror str) id $ runParser (evalStateT (amountp <* eof) mempty) "" $ T.pack $ (currency++) $ simplifySign str, nullsourcepos)
+    balanceerror str err = error' $ unlines
+      ["error: could not parse \""++str++"\" as balance amount"
+      ,showRecord record
+      ,"the balance rule is:      "++(fromMaybe "" $ mfieldtemplate "balance")
+      ,"the currency rule is:    "++(fromMaybe "unspecified" $ mfieldtemplate "currency")
+      ,"the default-currency is: "++fromMaybe "unspecified" mdefaultcurrency
+      ,"the parse error is:      "++show err
+      ]
 
     -- build the transaction
     t = nulltransaction{
@@ -675,12 +700,12 @@ transactionFromCsvRecord sourcepos rules record = t
       tcomment                 = T.pack comment,
       tpreceding_comment_lines = T.pack precomment,
       tpostings                =
-        [posting {paccount=account2, pamount=amount2, ptransaction=Just t}
-        ,posting {paccount=account1, pamount=amount1, ptransaction=Just t}
+        [posting {paccount=account1, pamount=amount1, ptransaction=Just t, pbalanceassertion=balance}
+        ,posting {paccount=account2, pamount=amount2, ptransaction=Just t}
         ]
       }
 
-getAmountStr :: CsvRules -> CsvRecord -> String
+getAmountStr :: CsvRules -> CsvRecord -> Maybe String
 getAmountStr rules record =
  let
    mamount    = getEffectiveAssignment rules record "amount"
@@ -689,17 +714,23 @@ getAmountStr rules record =
    render     = fmap (strip . renderTemplate rules record)
  in
   case (render mamount, render mamountin, render mamountout) of
-    (Just "", Nothing, Nothing) -> error' $ "amount has no value\n"++showRecord record
-    (Just a,  Nothing, Nothing) -> a
+    (Just "", Nothing, Nothing) -> Nothing
+    (Just a,  Nothing, Nothing) -> Just a
     (Nothing, Just "", Just "") -> error' $ "neither amount-in or amount-out has a value\n"++showRecord record
-    (Nothing, Just i,  Just "") -> i
-    (Nothing, Just "", Just o)  -> negateStr o
+    (Nothing, Just i,  Just "") -> Just i
+    (Nothing, Just "", Just o)  -> Just $ negateStr o
     (Nothing, Just _,  Just _)  -> error' $ "both amount-in and amount-out have a value\n"++showRecord record
     _                           -> error' $ "found values for amount and for amount-in/amount-out - please use either amount or amount-in/amount-out\n"++showRecord record
 
-negateIfParenthesised :: String -> String
-negateIfParenthesised ('(':s) | lastMay s == Just ')' = negateStr $ init s
-negateIfParenthesised s                               = s
+type CsvAmountString = String
+
+-- | Canonicalise the sign in a CSV amount string.
+-- Such strings can be parenthesized, which is equivalent to having a minus sign.
+-- Also they can end up with a double minus sign, which cancels out.
+simplifySign :: CsvAmountString -> CsvAmountString
+simplifySign ('(':s) | lastMay s == Just ')' = simplifySign $ negateStr $ init s
+simplifySign ('-':'-':s) = s
+simplifySign s = s
 
 negateStr :: String -> String
 negateStr ('-':s) = s
