@@ -81,6 +81,8 @@ import Control.Monad.State.Strict
 import qualified Data.Map.Strict as M
 import Data.Monoid
 import Data.Text (Text)
+import Data.String
+import Data.List
 import qualified Data.Text as T
 import Data.Time.Calendar
 import Data.Time.LocalTime
@@ -90,7 +92,7 @@ import Test.HUnit
 import Test.Framework
 import Text.Megaparsec.Error
 #endif
-import Text.Megaparsec hiding (parse)
+import Text.Megaparsec.Compat hiding (parse)
 import Text.Printf
 import System.FilePath
 
@@ -157,7 +159,7 @@ addJournalItemP =
 directivep :: MonadIO m => ErroringJournalParser m ()
 directivep = (do
   optional $ char '!'
-  choiceInState [
+  choice [
     includedirectivep
    ,aliasdirectivep
    ,endaliasesdirectivep
@@ -187,7 +189,7 @@ includedirectivep = do
       let curdir = takeDirectory (sourceName parentpos)
       filepath <- expandPath curdir filename `orRethrowIOError` (show parentpos ++ " locating " ++ filename)
       txt      <- readFileAnyLineEnding filepath `orRethrowIOError` (show parentpos ++ " reading " ++ filepath)
-      (ej1::Either (ParseError Char Dec) ParsedJournal) <-
+      (ej1::Either (ParseError Char MPErr) ParsedJournal) <-
         runParserT
            (evalStateT
               (choiceInState
@@ -201,7 +203,7 @@ includedirectivep = do
       either
         (throwError
           . ((show parentpos ++ " in included file " ++ show filename ++ ":\n") ++)
-          . show)
+          . parseErrorPretty)
         (return . journalAddFile (filepath, txt))
         ej1
   case ej of
@@ -215,6 +217,7 @@ newJournalWithParseStateFrom j = mempty{
   ,jparsedefaultcommodity = jparsedefaultcommodity j
   ,jparseparentaccounts   = jparseparentaccounts j
   ,jparsealiases          = jparsealiases j
+  ,jcommodities           = jcommodities j
   -- ,jparsetransactioncount = jparsetransactioncount j
   ,jparsetimeclockentries = jparsetimeclockentries j
   }
@@ -227,7 +230,7 @@ orRethrowIOError io msg =
     (Right <$> io)
     `C.catch` \(e::C.IOException) -> return $ Left $ printf "%s:\n%s" msg (show e)
 
-accountdirectivep :: JournalStateParser m ()
+accountdirectivep :: JournalParser m ()
 accountdirectivep = do
   string "account"
   lift (some spacenonewline)
@@ -237,7 +240,7 @@ accountdirectivep = do
   modify' (\j -> j{jaccounts = acct : jaccounts j})
 
 
-indentedlinep :: JournalStateParser m String
+indentedlinep :: JournalParser m String
 indentedlinep = lift (some spacenonewline) >> (rstrip <$> lift restofline)
 
 -- | Parse a one-line or multi-line commodity directive.
@@ -253,7 +256,7 @@ commoditydirectivep = try commoditydirectiveonelinep <|> commoditydirectivemulti
 --
 -- >>> Right _ <- rejp commoditydirectiveonelinep "commodity $1.00"
 -- >>> Right _ <- rejp commoditydirectiveonelinep "commodity $1.00 ; blah\n"
-commoditydirectiveonelinep :: Monad m => JournalStateParser m ()
+commoditydirectiveonelinep :: Monad m => JournalParser m ()
 commoditydirectiveonelinep = do
   string "commodity"
   lift (some spacenonewline)
@@ -292,20 +295,30 @@ formatdirectivep expectedsym = do
     else parserErrorAt pos $
          printf "commodity directive symbol \"%s\" and format directive symbol \"%s\" should be the same" expectedsym acommodity
 
-applyaccountdirectivep :: JournalStateParser m ()
+keywordp :: String -> JournalParser m ()
+keywordp = (() <$) . string . fromString
+
+spacesp :: JournalParser m ()
+spacesp = () <$ lift (some spacenonewline)
+
+-- | Backtracking parser similar to string, but allows varying amount of space between words
+keywordsp :: String -> JournalParser m ()
+keywordsp = try . sequence_ . intersperse spacesp . map keywordp . words
+
+applyaccountdirectivep :: JournalParser m ()
 applyaccountdirectivep = do
-  string "apply" >> lift (some spacenonewline) >> string "account"
+  keywordsp "apply account" <?> "apply account directive"
   lift (some spacenonewline)
   parent <- lift accountnamep
   newline
   pushParentAccount parent
 
-endapplyaccountdirectivep :: JournalStateParser m ()
+endapplyaccountdirectivep :: JournalParser m ()
 endapplyaccountdirectivep = do
-  string "end" >> lift (some spacenonewline) >> string "apply" >> lift (some spacenonewline) >> string "account"
+  keywordsp "end apply account" <?> "end apply account directive"
   popParentAccount
 
-aliasdirectivep :: JournalStateParser m ()
+aliasdirectivep :: JournalParser m ()
 aliasdirectivep = do
   string "alias"
   lift (some spacenonewline)
@@ -321,7 +334,7 @@ basicaliasp = do
   old <- rstrip <$> (some $ noneOf ("=" :: [Char]))
   char '='
   many spacenonewline
-  new <- rstrip <$> anyChar `manyTill` eolof  -- don't require a final newline, good for cli options
+  new <- rstrip <$> anyChar `manyTill` eolof  -- eol in journal, eof in command lines, normally
   return $ BasicAlias (T.pack old) (T.pack new)
 
 regexaliasp :: TextParser m AccountAlias
@@ -333,15 +346,15 @@ regexaliasp = do
   many spacenonewline
   char '='
   many spacenonewline
-  repl <- rstrip <$> anyChar `manyTill` eolof
+  repl <- anyChar `manyTill` eolof
   return $ RegexAlias re repl
 
-endaliasesdirectivep :: JournalStateParser m ()
+endaliasesdirectivep :: JournalParser m ()
 endaliasesdirectivep = do
-  string "end aliases"
+  keywordsp "end aliases" <?> "end aliases directive"
   clearAccountAliases
 
-tagdirectivep :: JournalStateParser m ()
+tagdirectivep :: JournalParser m ()
 tagdirectivep = do
   string "tag" <?> "tag directive"
   lift (some spacenonewline)
@@ -349,13 +362,13 @@ tagdirectivep = do
   lift restofline
   return ()
 
-endtagdirectivep :: JournalStateParser m ()
+endtagdirectivep :: JournalParser m ()
 endtagdirectivep = do
-  (string "end tag" <|> string "pop") <?> "end tag or pop directive"
+  (keywordsp "end tag" <|> keywordp "pop") <?> "end tag or pop directive"
   lift restofline
   return ()
 
-defaultyeardirectivep :: JournalStateParser m ()
+defaultyeardirectivep :: JournalParser m ()
 defaultyeardirectivep = do
   char 'Y' <?> "default year"
   lift (many spacenonewline)
@@ -364,7 +377,7 @@ defaultyeardirectivep = do
   failIfInvalidYear y
   setYear y'
 
-defaultcommoditydirectivep :: Monad m => JournalStateParser m ()
+defaultcommoditydirectivep :: Monad m => JournalParser m ()
 defaultcommoditydirectivep = do
   char 'D' <?> "default commodity"
   lift (some spacenonewline)
@@ -372,7 +385,7 @@ defaultcommoditydirectivep = do
   lift restofline
   setDefaultCommodityAndStyle (acommodity, astyle)
 
-marketpricedirectivep :: Monad m => JournalStateParser m MarketPrice
+marketpricedirectivep :: Monad m => JournalParser m MarketPrice
 marketpricedirectivep = do
   char 'P' <?> "market price"
   lift (many spacenonewline)
@@ -384,7 +397,7 @@ marketpricedirectivep = do
   lift restofline
   return $ MarketPrice date symbol price
 
-ignoredpricecommoditydirectivep :: JournalStateParser m ()
+ignoredpricecommoditydirectivep :: JournalParser m ()
 ignoredpricecommoditydirectivep = do
   char 'N' <?> "ignored-price commodity"
   lift (some spacenonewline)
@@ -392,7 +405,7 @@ ignoredpricecommoditydirectivep = do
   lift restofline
   return ()
 
-commodityconversiondirectivep :: Monad m => JournalStateParser m ()
+commodityconversiondirectivep :: Monad m => JournalParser m ()
 commodityconversiondirectivep = do
   char 'C' <?> "commodity conversion"
   lift (some spacenonewline)
@@ -470,7 +483,7 @@ test_transactionp = do
      nulltransaction{
       tdate=parsedate "2012/05/14",
       tdate2=Just $ parsedate "2012/05/15",
-      tstatus=Uncleared,
+      tstatus=Unmarked,
       tcode="code",
       tdescription="desc",
       tcomment=" tcomment1\n tcomment2\n ttag1: val1\n",
