@@ -5,7 +5,9 @@ Example:
 
 @
 #DATE
-#ACCT DOTS  # Each dot represents 15m, spaces are ignored
+#ACCT  DOTS  # Each dot represents 15m, spaces are ignored
+#ACCT  8    # numbers with or without a following h represent hours
+#ACCT  5m   # numbers followed by m represent minutes
 
 # on 2/1, 1h was spent on FOSS haskell work, 0.25h on research, etc.
 2/1
@@ -42,7 +44,7 @@ import Data.List (foldl')
 import Data.Maybe
 import Data.Text (Text)
 import Test.HUnit
-import Text.Megaparsec hiding (parse)
+import Text.Megaparsec.Compat hiding (parse)
 
 import Hledger.Data
 import Hledger.Read.Common
@@ -66,12 +68,12 @@ reader = Reader
 parse :: Maybe FilePath -> Bool -> FilePath -> Text -> ExceptT String IO Journal
 parse _ = parseAndFinaliseJournal timedotfilep
 
-timedotfilep :: JournalStateParser m ParsedJournal
+timedotfilep :: JournalParser m ParsedJournal
 timedotfilep = do many timedotfileitemp
                   eof
                   get
     where
-      timedotfileitemp :: JournalStateParser m ()
+      timedotfileitemp :: JournalParser m ()
       timedotfileitemp = do
         ptrace "timedotfileitemp"
         choice [
@@ -89,7 +91,7 @@ addTransactions ts j = foldl' (flip ($)) j (map addTransaction ts)
 -- biz.research .
 -- inc.client1  .... .... .... .... .... ....
 -- @
-timedotdayp :: JournalStateParser m [Transaction]
+timedotdayp :: JournalParser m [Transaction]
 timedotdayp = do
   ptrace " timedotdayp"
   d <- datep <* lift eolof
@@ -101,7 +103,7 @@ timedotdayp = do
 -- @
 -- fos.haskell  .... ..
 -- @
-timedotentryp :: JournalStateParser m Transaction
+timedotentryp :: JournalParser m Transaction
 timedotentryp = do
   ptrace "  timedotentryp"
   pos <- genericSourcePos <$> getPosition
@@ -125,26 +127,48 @@ timedotentryp = do
         }
   return t
 
-timedotdurationp :: JournalStateParser m Quantity
-timedotdurationp = try timedotnumberp <|> timedotdotsp
+timedotdurationp :: JournalParser m Quantity
+timedotdurationp = try timedotnumericp <|> timedotdotsp
 
--- | Parse a duration written as a decimal number of hours (optionally followed by the letter h).
+-- | Parse a duration of seconds, minutes, hours, days, weeks, months or years,
+-- written as a decimal number followed by s, m, h, d, w, mo or y, assuming h
+-- if there is no unit. Returns the duration as hours, assuming
+-- 1m = 60s, 1h = 60m, 1d = 24h, 1w = 7d, 1mo = 30d, 1y=365d.
 -- @
+-- 1.5
 -- 1.5h
+-- 90m
 -- @
-timedotnumberp :: JournalStateParser m Quantity
-timedotnumberp = do
-   (q, _, _, _) <- lift numberp
-   lift (many spacenonewline)
-   optional $ char 'h'
-   lift (many spacenonewline)
-   return q
+timedotnumericp :: JournalParser m Quantity
+timedotnumericp = do
+  (q, _, _, _) <- lift $ numberp Nothing
+  msymbol <- optional $ choice $ map (string . fst) timeUnits
+  lift (many spacenonewline)
+  let q' = 
+        case msymbol of
+          Nothing  -> q
+          Just sym ->
+            case lookup sym timeUnits of
+              Just mult -> q * mult  
+              Nothing   -> q  -- shouldn't happen.. ignore
+  return q'
+
+-- (symbol, equivalent in hours). 
+timeUnits =
+  [("s",2.777777777777778e-4)
+  ,("mo",5040) -- before "m"
+  ,("m",1.6666666666666666e-2)
+  ,("h",1)
+  ,("d",24)
+  ,("w",168)
+  ,("y",61320)
+  ]
 
 -- | Parse a quantity written as a line of dots, each representing 0.25.
 -- @
 -- .... ..
 -- @
-timedotdotsp :: JournalStateParser m Quantity
+timedotdotsp :: JournalParser m Quantity
 timedotdotsp = do
   dots <- filter (not.isSpace) <$> many (oneOf (". " :: [Char]))
   return $ (/4) $ fromIntegral $ length dots
