@@ -17,9 +17,6 @@ module Hledger.Reports.BalanceReport (
   BalanceReport,
   BalanceReportItem,
   balanceReport,
-  balanceReportValue,
-  mixedAmountValue,
-  amountValue,
   flatShowsExclusiveBalance,
 
   -- * Tests
@@ -32,7 +29,6 @@ import Data.Ord
 import Data.Maybe
 import Data.Time.Calendar
 import Test.HUnit
-import qualified Data.Text as T
 
 import Hledger.Data
 import Hledger.Read (mamountp')
@@ -92,6 +88,7 @@ balanceReport opts q j = (items, total)
                          dbg1 "accts" $
                          take 1 $ clipAccountsAndAggregate (queryDepth q) $ flattenAccounts accts
           | flat_ opts = dbg1 "accts" $
+                         maybesortflat $
                          filterzeros $
                          filterempty $
                          drop 1 $ clipAccountsAndAggregate (queryDepth q) $ flattenAccounts accts
@@ -100,6 +97,7 @@ balanceReport opts q j = (items, total)
                          drop 1 $ flattenAccounts $
                          markboring $
                          prunezeros $
+                         maybesorttree $
                          clipAccounts (queryDepth q) accts
           where
             balance     = if flat_ opts then aebalance else aibalance
@@ -107,6 +105,12 @@ balanceReport opts q j = (items, total)
             filterempty = filter (\a -> anumpostings a > 0 || not (isZeroMixedAmount (balance a)))
             prunezeros  = if empty_ opts then id else fromMaybe nullacct . pruneAccounts (isZeroMixedAmount . balance)
             markboring  = if no_elide_ opts then id else markBoringParentAccounts
+            maybesortflat | sort_amount_ opts = sortBy (maybeflip $ comparing balance)
+                          | otherwise = id
+              where
+                maybeflip = if normalbalance_ opts == Just NormalNegative then id else flip
+            maybesorttree | sort_amount_ opts = sortAccountTreeByAmount (fromMaybe NormalPositive $ normalbalance_ opts)
+                          | otherwise = id
       items = dbg1 "items" $ map (balanceReportItem opts q) accts'
       total | not (flat_ opts) = dbg1 "total" $ sum [amt | (_,_,indent,amt) <- items, indent == 0]
             | otherwise        = dbg1 "total" $
@@ -144,57 +148,6 @@ balanceReportItem opts q a
 --     items = [(a,a',n, headDef 0 bs) | ((a,a',n), bs) <- mbrrows]
 --     total = headDef 0 mbrtotals
 
--- | Convert all the amounts in a single-column balance report to
--- their value on the given date in their default valuation
--- commodities.
-balanceReportValue :: Journal -> Day -> BalanceReport -> BalanceReport
-balanceReportValue j d r = r'
-  where
-    (items,total) = r
-    r' =
-      dbg8 "known market prices" (jmarketprices j) `seq`
-      dbg8 "report end date" d `seq`
-      dbg8 "balanceReportValue"
-        ([(n, n', i, mixedAmountValue j d a) |(n,n',i,a) <- items], mixedAmountValue j d total)
-
-mixedAmountValue :: Journal -> Day -> MixedAmount -> MixedAmount
-mixedAmountValue j d (Mixed as) = Mixed $ map (amountValue j d) as
-
--- | Find the market value of this amount on the given date, in it's
--- default valuation commodity, based on recorded market prices.
--- If no default valuation commodity can be found, the amount is left
--- unchanged.
-amountValue :: Journal -> Day -> Amount -> Amount
-amountValue j d a =
-  case commodityValue j d (acommodity a) of
-    Just v  -> v{aquantity=aquantity v * aquantity a
-                ,aprice=aprice a
-                }
-    Nothing -> a
-
--- | Find the market value, if known, of one unit of this commodity (A) on
--- the given valuation date, in the commodity (B) mentioned in the latest
--- applicable market price. The latest applicable market price is the market
--- price directive for commodity A with the latest date that is on or before
--- the valuation date; or if there are multiple such prices with the same date,
--- the last parsed.
-commodityValue :: Journal -> Day -> CommoditySymbol -> Maybe Amount
-commodityValue j valuationdate c
-    | null applicableprices = dbg Nothing
-    | otherwise             = dbg $ Just $ mpamount $ last applicableprices
-  where
-    dbg = dbg8 ("using market price for "++T.unpack c)
-    applicableprices =
-      [p | p <- sortBy (comparing mpdate) $ jmarketprices j
-      , mpcommodity p == c
-      , mpdate p <= valuationdate
-      ]
-
-
-
-
-
-
 
 tests_balanceReport =
   let
@@ -213,8 +166,10 @@ tests_balanceReport =
   ,"balanceReport with no args on sample journal" ~: do
    (defreportopts, samplejournal) `gives`
     ([
-      ("assets","assets",0, mamountp' "$-1.00")
-     ,("assets:bank:saving","bank:saving",1, mamountp' "$1.00")
+      ("assets","assets",0, mamountp' "$0.00")
+     ,("assets:bank","bank",1, mamountp' "$2.00")
+     ,("assets:bank:checking","checking",2, mamountp' "$1.00")
+     ,("assets:bank:saving","saving",2, mamountp' "$1.00")
      ,("assets:cash","cash",1, mamountp' "$-2.00")
      ,("expenses","expenses",0, mamountp' "$2.00")
      ,("expenses:food","food",1, mamountp' "$1.00")
@@ -222,27 +177,22 @@ tests_balanceReport =
      ,("income","income",0, mamountp' "$-2.00")
      ,("income:gifts","gifts",1, mamountp' "$-1.00")
      ,("income:salary","salary",1, mamountp' "$-1.00")
-     ,("liabilities:debts","liabilities:debts",0, mamountp' "$1.00")
      ],
      Mixed [usd0])
 
   ,"balanceReport with --depth=N" ~: do
    (defreportopts{depth_=Just 1}, samplejournal) `gives`
     ([
-      ("assets",      "assets",      0, mamountp' "$-1.00")
-     ,("expenses",    "expenses",    0, mamountp'  "$2.00")
+     ("expenses",    "expenses",    0, mamountp'  "$2.00")
      ,("income",      "income",      0, mamountp' "$-2.00")
-     ,("liabilities", "liabilities", 0, mamountp'  "$1.00")
      ],
      Mixed [usd0])
 
   ,"balanceReport with depth:N" ~: do
    (defreportopts{query_="depth:1"}, samplejournal) `gives`
     ([
-      ("assets",      "assets",      0, mamountp' "$-1.00")
-     ,("expenses",    "expenses",    0, mamountp'  "$2.00")
+     ("expenses",    "expenses",    0, mamountp'  "$2.00")
      ,("income",      "income",      0, mamountp' "$-2.00")
-     ,("liabilities", "liabilities", 0, mamountp'  "$1.00")
      ],
      Mixed [usd0])
 
@@ -268,19 +218,30 @@ tests_balanceReport =
   ,"balanceReport with not:desc:" ~: do
    (defreportopts{query_="not:desc:income"}, samplejournal) `gives`
     ([
-      ("assets","assets",0, mamountp' "$-2.00")
-     ,("assets:bank","bank",1, Mixed [usd0])
-     ,("assets:bank:checking","checking",2,mamountp' "$-1.00")
-     ,("assets:bank:saving","saving",2, mamountp' "$1.00")
+      ("assets","assets",0, mamountp' "$-1.00")
+     ,("assets:bank:saving","bank:saving",1, mamountp' "$1.00")
      ,("assets:cash","cash",1, mamountp' "$-2.00")
      ,("expenses","expenses",0, mamountp' "$2.00")
      ,("expenses:food","food",1, mamountp' "$1.00")
      ,("expenses:supplies","supplies",1, mamountp' "$1.00")
      ,("income:gifts","income:gifts",0, mamountp' "$-1.00")
-     ,("liabilities:debts","liabilities:debts",0, mamountp' "$1.00")
      ],
      Mixed [usd0])
 
+  ,"balanceReport with period on a populated period" ~: do
+    (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 1) (fromGregorian 2008 1 2)}, samplejournal) `gives`
+     (
+      [
+       ("assets:bank:checking","assets:bank:checking",0, mamountp' "$1.00")
+      ,("income:salary","income:salary",0, mamountp' "$-1.00")
+      ],
+      Mixed [usd0])
+
+   ,"balanceReport with period on an unpopulated period" ~: do
+    (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 2) (fromGregorian 2008 1 3)}, samplejournal) `gives`
+     ([],Mixed [nullamt])
+
+
 
 {-
     ,"accounts report with account pattern o" ~:
@@ -407,7 +368,7 @@ Right samplejournal2 =
           tsourcepos=nullsourcepos,
           tdate=parsedate "2008/01/01",
           tdate2=Just $ parsedate "2009/01/01",
-          tstatus=Uncleared,
+          tstatus=Unmarked,
           tcode="",
           tdescription="income",
           tcomment="",
