@@ -1,4 +1,4 @@
-{-# LANGUAGE FlexibleInstances, ScopedTypeVariables #-}
+{-# LANGUAGE FlexibleInstances, ScopedTypeVariables, OverloadedStrings #-}
 {-|
 
 Multi-column balance reports, used by the balance command.
@@ -9,11 +9,10 @@ module Hledger.Reports.MultiBalanceReports (
   MultiBalanceReport(..),
   MultiBalanceReportRow,
   multiBalanceReport,
-  multiBalanceReportValue,
-  singleBalanceReport
+  singleBalanceReport,
 
   -- -- * Tests
-  -- tests_Hledger_Reports_MultiBalanceReport
+  tests_Hledger_Reports_MultiBalanceReport
 )
 where
 
@@ -22,11 +21,12 @@ import Data.Maybe
 import Data.Ord
 import Data.Time.Calendar
 import Safe
--- import Test.HUnit
+import Test.HUnit
 
 import Hledger.Data
 import Hledger.Query
 import Hledger.Utils
+import Hledger.Read (mamountp')
 import Hledger.Reports.ReportOptions
 import Hledger.Reports.BalanceReport
 
@@ -35,7 +35,7 @@ import Hledger.Reports.BalanceReport
 --
 -- 1. a list of each column's period (date span)
 --
--- 2. a list of row items, each containing:
+-- 2. a list of rows, each containing:
 --
 --   * the full account name
 --
@@ -43,7 +43,7 @@ import Hledger.Reports.BalanceReport
 --
 --   * the account's depth
 --
---   * the amounts to show in each column
+--   * a list of amounts, one for each column
 --
 --   * the total of the row's amounts
 --
@@ -53,14 +53,14 @@ import Hledger.Reports.BalanceReport
 --
 -- The meaning of the amounts depends on the type of multi balance
 -- report, of which there are three: periodic, cumulative and historical
--- (see 'BalanceType' and "Hledger.Cli.Balance").
+-- (see 'BalanceType' and "Hledger.Cli.Commands.Balance").
 newtype MultiBalanceReport =
   MultiBalanceReport ([DateSpan]
                      ,[MultiBalanceReportRow]
                      ,MultiBalanceReportTotals
                      )
 type MultiBalanceReportRow    = (AccountName, AccountName, Int, [MixedAmount], MixedAmount, MixedAmount)
-type MultiBalanceReportTotals = ([MixedAmount], MixedAmount, MixedAmount)
+type MultiBalanceReportTotals = ([MixedAmount], MixedAmount, MixedAmount) -- (Totals list, sum of totals, average of totals)
 
 instance Show MultiBalanceReport where
     -- use ppShow to break long lists onto multiple lines
@@ -90,7 +90,7 @@ singleBalanceReport opts q j = (rows', total)
 -- showing the change of balance, accumulated balance, or historical balance
 -- in each of the specified periods.
 multiBalanceReport :: ReportOpts -> Query -> Journal -> MultiBalanceReport
-multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow)
+multiBalanceReport opts q j = MultiBalanceReport (displayspans, sorteditems, totalsrow)
     where
       symq       = dbg1 "symq"   $ filterQuery queryIsSym $ dbg1 "requested q" q
       depthq     = dbg1 "depthq" $ filterQuery queryIsDepth q
@@ -155,7 +155,7 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
           dbg1 "displayedAccts" $
           (if tree_ opts then expandAccountNames else id) $
           nub $ map (clipOrEllipsifyAccountName depth) $
-          if empty_ opts then nub $ sort $ startAccts ++ postedAccts else postedAccts
+          if empty_ opts || (balancetype_ opts) == HistoricalBalance then nub $ sort $ startAccts ++ postedAccts else postedAccts
 
       acctBalChangesPerSpan :: [[(ClippedAccountName, MixedAmount)]] =
           dbg1 "acctBalChangesPerSpan"
@@ -168,7 +168,7 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
           [(a, map snd abs) | abs@((a,_):_) <- transpose acctBalChangesPerSpan] -- never null, or used when null...
 
       items :: [MultiBalanceReportRow] =
-          dbg1 "items"
+          dbg1 "items" $
           [(a, accountLeafName a, accountNameLevel a, displayedBals, rowtot, rowavg)
            | (a,changes) <- acctBalChanges
            , let displayedBals = case balancetype_ opts of
@@ -180,11 +180,48 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
            , empty_ opts || depth == 0 || any (not . isZeroMixedAmount) displayedBals
            ]
 
+      sorteditems :: [MultiBalanceReportRow] =
+        dbg1 "sorteditems" $
+        maybesort items
+        where
+          maybesort
+            | not $ sort_amount_ opts         = id
+            | accountlistmode_ opts == ALTree = sortTreeMultiBalanceReportRowsByAmount
+            | otherwise                       = sortFlatMultiBalanceReportRowsByAmount
+            where
+              -- Sort the report rows, representing a flat account list, by row total. 
+              sortFlatMultiBalanceReportRowsByAmount = sortBy (maybeflip $ comparing fifth6)
+                where
+                  maybeflip = if normalbalance_ opts == Just NormalNegative then id else flip
+
+              -- Sort the report rows, representing a tree of accounts, by row total at each level.
+              -- To do this we recreate an Account tree with the row totals as balances, 
+              -- so we can do a hierarchical sort, flatten again, and then reorder the  
+              -- report rows similarly. Yes this is pretty long winded. 
+              sortTreeMultiBalanceReportRowsByAmount rows = sortedrows
+                where
+                  anamesandrows = [(first6 r, r) | r <- rows]
+                  anames = map fst anamesandrows
+                  atotals = [(a,tot) | (a,_,_,_,tot,_) <- rows]
+                  nametree = treeFromPaths $ map expandAccountName anames
+                  accounttree = nameTreeToAccount "root" nametree
+                  accounttreewithbals = mapAccounts setibalance accounttree
+                    where
+                      -- this error should not happen, but it's ugly TODO 
+                      setibalance a = a{aibalance=fromMaybe (error "sortTreeMultiBalanceReportRowsByAmount 1") $ lookup (aname a) atotals}
+                  sortedaccounttree = sortAccountTreeByAmount (fromMaybe NormalPositive $ normalbalance_ opts) accounttreewithbals
+                  sortedaccounts = drop 1 $ flattenAccounts sortedaccounttree
+                  sortedrows = [ 
+                    -- this error should not happen, but it's ugly TODO 
+                    fromMaybe (error "sortTreeMultiBalanceReportRowsByAmount 2") $ lookup (aname a) anamesandrows
+                    | a <- sortedaccounts 
+                    ]
+
       totals :: [MixedAmount] =
           -- dbg1 "totals" $
           map sum balsbycol
           where
-            balsbycol = transpose [bs | (a,_,_,bs,_,_) <- items, not (tree_ opts) || a `elem` highestlevelaccts]
+            balsbycol = transpose [bs | (a,_,_,bs,_,_) <- sorteditems, not (tree_ opts) || a `elem` highestlevelaccts]
             highestlevelaccts     =
                 dbg1 "highestlevelaccts"
                 [a | a <- displayedAccts, not $ any (`elem` displayedAccts) $ init $ expandAccountName a]
@@ -196,16 +233,54 @@ multiBalanceReport opts q j = MultiBalanceReport (displayspans, items, totalsrow
       dbg1 s = let p = "multiBalanceReport" in Hledger.Utils.dbg1 (p++" "++s)  -- add prefix in this function's debug output
       -- dbg1 = const id  -- exclude this function from debug output
 
--- | Convert all the amounts in a multi-column balance report to their
--- value on the given date in their default valuation commodities
--- (which are determined as of that date, not the report interval dates).
-multiBalanceReportValue :: Journal -> Day -> MultiBalanceReport -> MultiBalanceReport
-multiBalanceReportValue j d r = r'
-  where
-    MultiBalanceReport (spans, rows, (coltotals, rowtotaltotal, rowavgtotal)) = r
-    r' = MultiBalanceReport
-         (spans,
-          [(acct, acct', depth, map convert rowamts, convert rowtotal, convert rowavg) | (acct, acct', depth, rowamts, rowtotal, rowavg) <- rows],
-          (map convert coltotals, convert rowtotaltotal, convert rowavgtotal))
-    convert = mixedAmountValue j d
 
+tests_multiBalanceReport =
+  let
+    (opts,journal) `gives` r = do
+      let (eitems, etotal) = r
+          (MultiBalanceReport (_, aitems, atotal)) = multiBalanceReport opts (queryFromOpts nulldate opts) journal
+          showw (acct,acct',indent,lAmt,amt,amt') = (acct, acct', indent, map showMixedAmountDebug lAmt, showMixedAmountDebug amt, showMixedAmountDebug amt')
+      assertEqual "items" (map showw eitems) (map showw aitems)
+      assertEqual "total" (showMixedAmountDebug etotal) ((\(_, b, _) -> showMixedAmountDebug b) atotal) -- we only check the sum of the totals
+    usd0 = usd 0
+    amount0 = Amount {acommodity="$", aquantity=0, aprice=NoPrice, astyle=AmountStyle {ascommodityside = L, ascommodityspaced = False, asprecision = 2, asdecimalpoint = Just '.', asdigitgroups = Nothing}, amultiplier=False}
+  in [
+   "multiBalanceReport with no args on null journal" ~: do
+   (defreportopts, nulljournal) `gives` ([], Mixed [nullamt])
+
+   ,"multiBalanceReport with -H on a populated period" ~: do
+    (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 1) (fromGregorian 2008 1 2), balancetype_=HistoricalBalance}, samplejournal) `gives`
+     (
+      [
+       ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+      ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+      ],
+      Mixed [usd0])
+
+   ,"multiBalanceReport tests the ability to have a valid history on an empty period" ~: do
+    (defreportopts{period_= PeriodBetween (fromGregorian 2008 1 2) (fromGregorian 2008 1 3), balancetype_=HistoricalBalance}, samplejournal) `gives`
+     (
+      [
+       ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+      ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+      ],
+      Mixed [usd0])
+
+   ,"multiBalanceReport tests the ability to have a valid history on an empty period (More complex)" ~: do
+    (defreportopts{period_= PeriodBetween (fromGregorian 2009 1 1) (fromGregorian 2009 1 2), balancetype_=HistoricalBalance}, samplejournal) `gives`
+     (
+      [
+      ("assets:bank:checking","checking",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+      ,("assets:bank:saving","saving",3, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=1}])
+      ,("assets:cash","cash",2, [mamountp' "$-2.00"], mamountp' "$-2.00",Mixed [amount0 {aquantity=(-2)}])
+      ,("expenses:food","food",2, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=(1)}])
+      ,("expenses:supplies","supplies",2, [mamountp' "$1.00"], mamountp' "$1.00",Mixed [amount0 {aquantity=(1)}])
+      ,("income:gifts","gifts",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+      ,("income:salary","salary",2, [mamountp' "$-1.00"], mamountp' "$-1.00",Mixed [amount0 {aquantity=(-1)}])
+      ],
+      Mixed [usd0])
+  ]
+
+tests_Hledger_Reports_MultiBalanceReport :: Test
+tests_Hledger_Reports_MultiBalanceReport = TestList
+  tests_multiBalanceReport
